

/**
 * Minimizer contains the data for the minimization of one function
 * on a given domain, subject to a system of constraints.
 */
class Minimizer {
public:
	// lower endpoints of interval.
	double* xmin;

	// upper endpoints of interval.
	double* xmax;

	// numerically determined coordinates of minimum.
	double* x;

	/**
	 * count number of times to repeat minimization.
	 * numargs size of arrays xmin, xmax, x;
	 * nconstr number of constraints.
	 */
	int count,numargs,nconstr;

	/**
	 * func function to be minimized.
	 * cFunc constraints to be used.
	 */
	void (*func)(int numargs,int whichFn,double* x,double* ret,void*);

	/**
	 * Each constraint is written in the form
	 * 	F < 0,
	 * The function cFunc sets *ret = F(p), so constraint is met
	 * iff *ret < 0.
	 * @param which = 1,...,nconstr, the constraint number
	 */
	void (*cFunc)(int numargs,int which,double* x,double* ret,void*);

	Minimizer(int count, int numargs, int nconstr, 
		double* xmin, double* xmax);

	/** carries out the optimization of the data. Sets x and
	 * returns the minimum value.
	 * @param randomize true to optimize count times on random input
	 * @param randomize false to optimize once on initial x input.
	 */
	double optimize(int randomize);

	// calls optimize, and then outputs a report.
	void coutReport();
};




