#include "Minimizer.h"
#include "kepfn.h"

#define amax 58

double e(double x, double y, int i, int j){
  static const double PI = 3.14159;
  return(cos(2.0*PI*((double)(i)*(y*2.0/sqrt(3.0)) + 
		     (double)(j)*(x-y*sqrt(1.0/3.0)))));
}


int size(int i,int j) {
	return i*i+j*j-i*j;
}

double  distR2(double x,double y,double x1,double y1) {
	double u = (x-x1) + (y-y1)/2.0;
	double v = (y-y1);
	return u*u + v*v*3.0/4.0;
}


void constraintCorner(int,int which,double* y,double* ret,void*) {
	static const double radius = 1.0/(2.0*5.0);
	static const double fr2 = 4.0*radius*radius;
	int i=0,j=0;
	switch(which) {
	case 1 : i=0; j=0; break;
	case 2 : i=0; j=1; break;
	case 3 : i=1; j=0; break;
	case 4 : i=1; j=1; break;
	}
	*ret = fr2- distR2(y[0],y[1],i,j);
}


double cx[amax];
/**
 * x1 and y1 between 0 and 1.
 */
double fn(double x1,double y1) {
	double x  = x1+y1/2.0;
	double y = y1*sqrt(3.0)/2.0; 
	double s = 1.0;
	for (int i=0;i<10;i++)
	for (int j=-10;j<10;j++) {
		if (i==0 && j<=0)
			continue;
		if (size(i,j)>=amax) continue;
		s += e(x,y,i,j)*cx[size(i,j)];
	}
	return -s;

}


/**
 * testing of volan inequality.
 */
void testFunction2(int, int, double* x,double* ret,void*) {
	*ret = fn(x[0],x[1]);
}


void runCH() {
	for (int i=0;i<amax;i++) cx[i]=0;
/*
cx[1] =                           1.890351;
cx[3]  =                          1.513786;
cx[4]   =                         1.323324;
cx[7]    =                        0.823231;
cx[9]     =                       0.583381;
cx[12]     =                      0.277222;
cx[13]      =                     0.222585;
cx[16]       =                    0.103946;
cx[19]        =                   0.010037;
cx[31]         =                  0.001021;;
cx[36]          =                 0.003661;
cx[37]           =                0.025990;
cx[39]            =               0.012042;
cx[43]             =              0.008610;
cx[48]              =             0.023825;
cx[49]               =            0.016379;
cx[52]                =           0.002891;
cx[57]                 =          0.009277;
*/
cx[1               ]=             1.907189;
cx[3              ]  =            1.535438;
cx[4             ]    =           1.351420;
cx[7            ]      =          0.830369;
cx[9           ]        =         0.587326;
cx[12         ]          =        0.277615;
cx[13        ]            =       0.217573;
cx[16       ]              =      0.092811;
cx[19      ]                =     0.005150;
cx[37     ]                  =    0.024177;
cx[39    ]                    =   0.006985;
cx[43   ]                      =  0.004223;
cx[48  ]                        = 0.020969;
cx[49 ]                          =0.012328;
cx[52]                      =     0.002096;
cx[57]                       =    0.004550;
	double xmin[2]= {0,0};
	double xmax[2]= {1,1};
	Minimizer M(50,2,4,xmin,xmax);
	M.func = testFunction2;
	M.cFunc = constraintCorner;
	M.optimize(1);
	M.coutReport();
}

