#include "Minimizer.h"
#include "kepfn.h"
#include <iomanip.h>

/**
  * INEQUALITIES ADDED IN THE 2001 revision of the Dodecahedral conjecture,
  * by Hales and McLaughlin.
  */


double seanT = 1.25841;
double seantauConstant = 0.42755;

double squander(double y0,double y1,double y2,double y3,double y4,double y5) {
		return volq(y0,y1,y2,y3,y4,y5,seanT) -
			seantauConstant*solid(y0,y1,y2,y3,y4,y5);
}



void testLP0(int,int,double* y,double* ret,void*) {
	*ret = -volq(y[0],y[1],y[2],y[3],y[4],y[5],seanT)+volan(y[0],y[1],y[2],y[3],y[4],y[5])
		+0.00051;
}

void runtestLP0() {
	double eM=seanT*2.0;
	double xmin[6]= {2,2,2,2,2,2};

	double xmax[6]= {eM,eM,eM,eM,eM,eM};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testLP0;
	cout << "testLP0:" << endl;
	M.coutReport();
	cout << "end-testLP0" << endl << endl;
}

void testLP1(int,int,double* y,double* ret,void*) {
	*ret = qy(y[0],y[1],y[3],seanT)+0.01*y[0]+0.0135*y[1]+0.0135*y[2]-0.0735;
}

void runtestLP1() {
	double eM=seanT*2.0;
	double xmin[6]= {2,2,2};

	double xmax[6]= {eM,eM,eM};
	Minimizer M(50,3,0,xmin,xmax);
	M.func = testLP1;
	cout << "testLP:" << endl;
	M.coutReport();
	cout << "end-testLP" << endl << endl;
}

void testLP2(int,int,double* y,double* ret,void*) {
	*ret = squander(y[0],y[1],y[2],y[3],y[4],y[5])
			+squander(y[6],y[1],y[2],y[3],y[7],y[8])-0.0436;
}

void constraintLP2(int,int which,double* y,double* ret,void*) {
		*ret= -crossdiag(y)+2.85;
}

void runtestLP2() {
	double eM=seanT*2.0;
	double xmin[9]= {2,2,2,2.85,2,2,2,2,2};

	double xmax[9]= {eM,eM,eM,3.2,eM,eM,eM,eM,eM};
	Minimizer M(50,9,1,xmin,xmax);
	M.func = testLP2;
	M.cFunc = constraintLP2;
	cout << "testLP2:" << endl;
	M.coutReport();
	cout << "end-testLP2" << endl << endl;
}

void testLP3(int,int,double* y,double* ret,void*) {
	*ret = dih(y[0],y[1],y[2],y[3],y[4],y[5]) 
		- 0.23* y[0] + 0.47 *y[1] + 0.47* y[2] - 0.7 *y[3] 
		+ 0.285* y[4] + 0.285* y[5] - 2.384 ;
}

void testLP4(int,int,double* y,double* ret,void*) {
	*ret = -dih(y[0],y[1],y[2],y[3],y[4],y[5]) 
		+ 0.54* y[0] - 0.15* y[1]- 0.15 *y[2] 
		+ 0.76 *y[3] -0.156* y[4] - 0.156* y[5] - 0.134 ;
}

void testLP5(int,int,double* y,double* ret,void*) {
	*ret = -dih(y[0],y[1],y[2],y[3],y[4],y[5]) 
		+ 0.544* y[0] - 0.169* y[1]- 0.169 *y[2] 
		+ 0.706 *y[3] -0.18* y[4] - 0.18* y[5] + 0.164 ;
}


void testLP7(int,int,double* y,double* ret,void*) {
	*ret = 1.0*solid(y[0],y[1],y[2],y[3],y[4],y[5]) +0.401*y[0] +0.401* y[1] +0.401*y[2] 
		-0.194*y[3] -0.194*y[4] - 0.194*y[5] -1.7911  ;
}

void testLP8(int,int,double* y,double* ret,void*) {
	*ret =  1.0 *squander(y[0],y[1],y[2],y[3],y[4],y[5]) -0.026 *y[0] -0.026 *y[1] -0.026 *y[2] 
		-0.039* y[3] -0.039 *y[4] -0.039* y[5] +0.3945 ;
}

void testLP9(int,int,double* y,double* ret,void*) {
	*ret =  1.0*dih(y[0],y[1],y[2],y[3],y[4],y[5]) 
		-0.328*y[0] +0.396* y[1] + 0.396 *y[2] + 0.396 *y[4] + 0.396* y[5] - 4.129  ;
}

void testLP10(int,int,double* y,double* ret,void*) {
	*ret = 1.0*dih(y[0],y[1],y[2],y[3],y[4],y[5]) 
		-0.335*y[0] +0.51*y[1] +0.51*y[2] +0.404*y[4] +0.404*y[5] -4.906   ;
}

void testLP11(int,int,double* y,double* ret,void*) {
	*ret =  -1.0* dih(y[0],y[1],y[2],y[3],y[4],y[5]) +0.362* y[0] -0.225* y[1] -0.088* y[2] +0.7068* y[3]
            -0.381* y[4] +0.1495* y[5] + 0.2352409  ;
}

void testLP12(int,int,double* y,double* ret,void*) {
	*ret = +1.0 *dih(y[0],y[1],y[2],y[3],y[4],y[5]) -0.112* y[0] +0.679* y[1] +0.3485* y[2] -0.606* y[3]
            + 0.428* y[4] + 0.004* y[5] - 2.778 ;
}


void runtestLP345() {
	double eM=seanT*2.0;
	double xmin[6]= {2,2,2,2,2,2};

	double xmax[6]= {eM,eM,eM,eM,eM,eM};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testLP3;
	cout << "testLP3:" << endl;
	M.coutReport();
	cout << "end-testLP3" << endl << endl;
	M.func = testLP4;
	cout << "testLP4:" << endl;
	M.coutReport();
	cout << "end-testLP4" << endl << endl;
	M.func = testLP5;
	cout << "testLP5:" << endl;
	M.coutReport();
	cout << "end-testLP5" << endl << endl;

	M.func = testLP7;
	cout << "testLP7:" << endl;
	M.coutReport();
	cout << "end-testLP7" << endl << endl;

	M.func = testLP8;
	cout << "testLP8:" << endl;
	M.coutReport();
	cout << "end-testLP8" << endl << endl;

}

void runtestLP9_10() {
	double eM=seanT*2.0;
	double xmin[6]= {2,2,2,eM,2,2};

	double xmax[6]= {eM,eM,eM,eM,eM,eM};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testLP9;
	cout << "testLP9:" << endl;
	M.coutReport();
	//cout << "end-testLP9" << endl << endl;
	xmin[3]=xmax[3]=2.85;
	Minimizer M2(50,6,0,xmin,xmax);
	M2.func = testLP10;
	cout << "testLP10:" << endl;
	M2.coutReport();
	//cout << "end-testLP10" << endl << endl;
}

void runtestLP11_12() {
	double eM=seanT*2.0;
	double xmin[6]= {2,2,2,2,eM,2};

	double xmax[6]= {eM,eM,eM,eM,2.85,eM};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testLP11;
	cout << "testLP11:" << endl;
	M.coutReport();
	//cout << "end-testLP11" << endl << endl;

	M.func = testLP12;
	cout << "testLP12:" << endl;
	M.coutReport();
	//cout << "end-testLP12" << endl << endl;

}

// now for inequalities used with \ref{dist2}

void test1a(int,int,double* y,double* ret,void*) {
	double d0 = dih(y[0],y[1],y[2],y[3],y[4],y[5]);
	double beta0 = beta(arc(y[1],2.0,2.0),arc(y[0],y[1],y[5]));
	*ret = d0-1.2;
}

void test1b(int,int,double* y,double* ret,void*) {
	double d0 = dih(y[0],y[1],y[2],y[3],y[4],y[5]);
	double beta0 = beta(arc(y[1],2.0,2.0),arc(y[0],y[1],y[5]));
	*ret = 1.2-beta0;
}

void run1() {
	double xmin[6]={2,2,2,3.2,2,2.7};
	double eM = getEdgeMax();
	double s8 = sqrt(8.0);
	double xmax[6]={eM,eM,eM,3.2,3.2,s8};
	Minimizer M(40,6,0,xmin,xmax);
	M.func = test1a;
	cout << "test 1a:" << endl;
	M.coutReport();
	cout << "end test1a " << endl << endl;
	M.func = test1b;
	cout << "test 1b:" << endl;
	M.coutReport();
	cout << "end test1b " << endl << endl;
}

void test2(int,int,double* y,double* ret,void*) {
	double alpha = arc(y[0],y[2],y[4]);
	double beta = arc(y[0],y[1],y[5]);
	//double beta = arc(2,2,2.7);
	double gamma = arc(y[1],y[2],y[3]);
	double rhs = -(cos(alpha)*cos(beta)-cos(gamma))/(cos(beta)*sin(alpha));
	double theta = atan(rhs);
	*ret = /*arc(getEdgeMax(),y[0],2.0)- */0.816 - theta;
	*ret = tan(0.816) - rhs/sin(alpha);
	
}

void run2() {
	double xmin[6]={2,2,2,3.2,2,2};
	double eM = getEdgeMax();
	double s8 = sqrt(8.0);
	double xmax[6]={eM,eM,eM,3.2,3.2,2.7};
	Minimizer M(40,6,0,xmin,xmax);
	M.func = test2;
	cout << "test 2:" << endl;
	M.coutReport();
	cout << "end test2 " << endl << endl;
}

void test3a(int,int,double* y,double* ret,void*) {
	double d1=dih(y[0],y[1],y[2],y[3],y[4],y[5]);
	double d2p=dih(y[0],y[1],2,2,2,y[5])+0.1;
	*ret = d1-d2p;
}

void test3b(int,int,double* y,double* ret,void*) {
	double d2=dih(y[0],y[1],y[2],2,2,y[5]);
	double d2p=dih(y[0],y[1],2,2,2,y[5])+0.1;
	*ret = -d2+d2p;
}


void run3() {
	double xmin[6]={2,2,2,3.2,2.91,2};
	double eM = getEdgeMax();
	double s8 = sqrt(8.0);
	double xmax[6]={eM,eM,eM,3.2,3.2,s8};
	Minimizer M(40,6,0,xmin,xmax);
	M.func = test3a;
	cout << "test 3a:" << endl;
	M.coutReport();
	cout << "end test3a " << endl << endl;
	M.func = test3b;
	cout << "test 3b:" << endl;
	M.coutReport();
	cout << "end test3b " << endl << endl;
}





void runCH() {
	//runtestLP0();
	//runtestLP1();
	//runtestLP2();
	//runtestLP345();
	//runtestLP9_10();
	//runtestLP11_12();
	//run1();
	//run2();
	run3();
	
}
