#include "Minimizer.h"
#include "kepfn.h"
#include <iomanip.h>

/**
 * testing of volan inequality.
 */
void testFunctionX(int, int, double* x,double* ret,void*) {
	*ret = volan(x[0],x[1],x[2],x[3],x[4],x[5])-
		volt(x[0],x[1],x[2],x[3],x[4],x[5])+0.00051;
}

double runX() {
	double xmin[6]= {2,2,2,2,2,2};
	double xmax[6]= {2.5,2.5,2.5,2.5,2.5,2.5};
	Minimizer M(10,6,0,xmin,xmax);
	M.func = testFunctionX;
	return M.optimize(1);
}

/**
  * ch2.5A, Bounds on the dihedral angle
  * minimum of dih(qrtet)
  */
void testCH25A1(int,int,double* y,double* ret,void*) {
	*ret = dih(y[0],y[1],y[2],y[3],y[4],y[5]) - 0.805;
}

void runCH25A1() {
	double xmin[6]= {2,2,2,2,2,2};
	double eM = getEdgeMax();
	double xmax[6]= {eM,eM,eM,eM,eM,eM};
	Minimizer M(10,6,0,xmin,xmax);
	M.func = testCH25A1;
	cout << "testCH25A1:" << endl;
	M.coutReport();
	cout << "end-testCH25A1" << endl << endl;
}

/**
  * ch2.5A, Bounds on the dihedral angle
  * minimum of dih(not-qrtet)
  */
void testCH25A2(int,int,double* y,double* ret,void*) {
	*ret = dih(y[0],y[1],y[2],y[3],y[4],y[5]) - 1.148;
}

void runCH25A2() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,eM,2,2};
	double xmax[6]= {eM,eM,eM,eM,eM,eM};
	Minimizer M(10,6,0,xmin,xmax);
	M.func = testCH25A2;
	cout << "testCH25A2:" << endl;
	M.coutReport();
	cout << "end-testCH25A2" << endl << endl;
}

/**
  * ch2.5A, Bounds on the dihedral angle
  * max of dih(qrtet)
  */
void testCH25A3(int,int,double* y,double* ret,void*) {
	*ret = -dih(y[0],y[1],y[2],y[3],y[4],y[5]) +1.97;
}

void runCH25A3() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,2,2,2};
	double xmax[6]= {eM,eM,eM,eM,eM,eM};
	Minimizer M(10,6,0,xmin,xmax);
	M.func = testCH25A3;
	cout << "testCH25A3:" << endl;
	M.coutReport();
	cout << "end-testCH25A3" << endl << endl;
}

/**
  * ch2.5A, Bounds on the dihedral angle
  * max of dih(qrtet)
  */
void testCH25A4(int,int,double* y,double* ret,void*) {
	*ret = -2.0*dih(y[0],y[1],y[2],y[3],y[4],y[5]) +3.34;
}

void runCH25A4() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,2,eM,2};
	double xmax[6]= {eM,eM,eM,eM,eM,eM};
	Minimizer M(10,6,0,xmin,xmax);
	M.func = testCH25A4;
	cout << "testCH25A4:" << endl;
	M.coutReport();
	cout << "end-testCH25A4" << endl << endl;
}

/**
  * Test code for inequality ch2.5B, tau(p,q) table.
  * Minimize what is squandered over p triangles and q quads at a vertex.
  * @param triangle: global variable containing triples {i,j,k} of vertex indices of triangles.
  * @param numTriangle: length of triangle is 3*numTriangle
  * @param longEdge: global, edges of length eM-sqrt8.
  * @param less2PiConstraint;
  *
  * [ FILL IN.... CODE NOT WRITTEN YET.... ]
  */

/**
  * Inequality ch2.6A, tau_R(V(eM/2)) > 0.03464
  * variable ordering as in crossdiag of kepfn.h
  */
void testCH26A(int,int,double* y,double* ret,void*) {
	*ret = taut(y[0],y[1],y[2],y[3],y[4],y[5]) +
		   taut(y[6],y[1],y[2],y[3],y[7],y[8]) 
		   - (L(y[0])+L(y[1])+L(y[2])+L(y[6]))
			- 0.03464;
}

void constraintCH26A(int,int,double* y,double* ret,void*) {
	*ret = -crossdiag(y) + safesqrt(8.0);
}

void runCH26A() {
	double eM = getEdgeMax();
	double xmin[9]= {2,2,2,sqrt(8.0),2,2,  2,2,2};
	double xmax[9]= {eM,eM,eM,3.0,eM,eM,   2,2,2};
	Minimizer M(20,9,1,xmin,xmax);
	M.func = testCH26A;
	M.cFunc = constraintCH26A;
	cout << "testCH26A:" << endl;
	M.coutReport();
	cout << "crossdiag = " << crossdiag(M.x) << endl;
	cout << "end-testCH26A" << endl << endl;
}

/**
  * Inequality ch2.6B, tau_flat(V(eM/2)) > 0.03464/2.0
  */
void testCH26B(int,int,double* y,double* ret,void*) {
	*ret = taut(y[0],y[1],y[2],y[3],y[4],y[5]) +
			M(y[3])-L(y[0])-L(y[1])/2.0-L(y[2])/2.0
		 - 0.01732;
}

void runCH26B() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,eM,2,2};
	double xmax[6]= {eM,eM,eM,sqrt(8.0),eM,eM};
	Minimizer M(20,6,0,xmin,xmax);
	M.func = testCH26B;
	cout << "testCH26B:" << endl;
	M.coutReport();
	cout << "end-testCH26B" << endl << endl;
}

/**
  * CHAPTER 3 INEQUALITIES
  */

/**
  * Inequality ch3.2A, dih > 2Pi/5
  */
void testCH32A(int,int,double* y,double* ret,void*) {
	double pi = atan(1.0)*4.0;
	*ret = dih(y[0],y[1],y[2],y[3],y[4],y[5]) -2.0*pi/5.0;
			
}

void runCH32A() {
	double eM = getEdgeMax();
	double xmin[6]= {eM,2,2,2,2,2};
	double xmax[6]= {sqrt(8.0),2.1,2.1,2.1,2.1,2.1};
	Minimizer M(20,6,0,xmin,xmax);
	M.func = testCH32A;
	cout << "testCH32A:" << endl;
	M.coutReport();
	cout << "end-testCH32A" << endl << endl;
}

/**
  * function ell(S) of ch3.4C
  */
double ell(double y1, double y2, double y3, double y4, double y5, double y6) {
	static int testcode=1;
	if (testcode) {
	double x1=4.0, x2=4.1, x3=4.2, x4=4.3, x5=4.4, x6=4.5;
	double delta4 = -x2*x3 - x1*x4 + x2*x5 + x3*x6 -x5*x6 +x1*(-x1+x2+x3-x4+x5+x6);
	double delta6 = -x1*x2 - x4*x5 + x1*x4 + x2*x5 -x3*x6 +x3*(x1+x2-x3+x4+x5-x6);
	if (delta4!=18.32) cout << "delta4 error: " << delta4 << endl;
	if (delta6!=15.04) cout << "delta6 error: " << delta6 << endl;
	testcode=0;
	}
	double x1 = y1*y1;
	double x2 = y2*y2;
	double x3 = y3*y3;
	double x4 = y4*y4;
	double x5 = y5*y5;
	double x6 = y6*y6;
	double u135 = uT(x1,x3,x5);
	double delta4 = -x2*x3 - x1*x4 + x2*x5 + x3*x6 -x5*x6 +x1*(-x1+x2+x3-x4+x5+x6);
	double delta6 = -x1*x2 - x4*x5 + x1*x4 + x2*x5 -x3*x6 +x3*(x1+x2-x3+x4+x5-x6);
	double trunc = getTrunc();
	double phit = phi(trunc,trunc) - 1.0/(3.0*getDtet());
	double b1 = phit - A(y1/2.0);
	double b2 = phit - A(y2/2.0);
	double b3 = phit - A(y3/2.0);
	return -b1*y1*delta6 + b2*y2*u135 - b3*y3*delta4;
}

/**
  * Inequality ch3.4C, ell > 0 
  */
void testCH34C(int,int,double* y,double* ret,void*) {
	*ret = ell(y[0],y[1],y[2],y[3],y[4],y[5]);		
}

void testCH34Cq(int,int,double* y,double* ret,void*) {
	double a = ell(y[0],y[1],y[2],y[3],y[4],y[5]);
	double b = ell(y[0],y[2],y[1],y[3],y[5],y[4]);
	*ret = (a > b ? a : b);		
}

void constraintCH34C(int,int which,double* y,double* ret,void*) {
	if (which==1) {
		*ret = y[4]-y[5];
		return;
	}
	if (which==2) {
		*ret =-delta(y[0]*y[0],y[1]*y[1],y[2]*y[2],y[3]*y[3],y[4]*y[4],y[5]*y[5]);
		return;
	}
	*ret = 3.14159/2.0 -dih(y[0],y[1],y[2],y[3],y[4],y[5]);
}

// for test purposes only...
void constraintCH34Ctest(int,int,double* y,double* ret,void*) {
		*ret= -(y[4]-y[5]);
		*ret = ell(y[0],y[2],y[1],y[3],y[5],y[4]);
}

double DquoinDx5(double y1,double y2,double y5) {
	double a = y1/2.0;
	double b = eta(y1,y2,y5);
	double c = getTrunc();
	if (b>c) return 0.0;
	double c2b2 = c*c-b*b;
	double b2a2 = b*b-a*a;
	double DquoinDb = -a*(c2b2*sqrt(c2b2))/(3.0*sqrt(b2a2)*b);
	//cout << "DquoinDb " << DquoinDb << endl;
	double x1=y1*y1;
	double x2=y2*y2;
	double x5=y5*y5;
	double u = uT(x1,x2,x5);
	double DbDx5 = (1.0/2.0)*(1.0/b)*x1*x2*(-x1+x2+x5)*(x1-x2+x5)/(u*u);
	//cout << "DbDx5 " << DbDx5 << endl;
	return DquoinDb*DbDx5;
}

// ch3.4C quoin calculation.
void testCH34C2(int,int,double* y,double* ret,void*) {
	*ret = (DquoinDx5(y[0],y[1],y[2])+DquoinDx5(y[1],y[0],y[2]))*
			uT(y[0]*y[0],y[1]*y[1],y[2]*y[2]);	
}

void testCH34C3(int,int,double* y,double* ret,void*) {
	*ret = ell(y[0],y[1],y[2],y[3],y[4],y[5])+
		safesqrt(delta(y[0]*y[0],y[1]*y[1],y[2]*y[2],y[3]*y[3],y[4]*y[4],y[5]*y[5]))*
		uT(y[0]*y[0],y[1]*y[1],y[4]*y[4])*
			(DquoinDx5(y[0],y[1],y[4])+DquoinDx5(y[1],y[0],y[2]));
}

void constraintDelta(int,int,double* y,double* ret,void*) {
		*ret= -delta(y[0]*y[0],y[1]*y[1],y[2]*y[2],y[3]*y[3],y[4]*y[4],y[5]*y[5]);
}

void runCH34C() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,0.001,2,3.2};
	// upper bound on y[3] comes from triangle ineq on y[1],y[2],y[3].
	double xmax[6]= {eM,eM,eM,2.0*eM,3.0,3.2};
	Minimizer M(50,6,2,xmin,xmax);
	M.func = testCH34C;
	//M.cFunc = constraintDelta;
	M.cFunc = constraintCH34C;
	//cout << "quoin value " << DquoinDx5(2.1,2.2,sqrt(4.3)) << endl;
	cout << "testCH34C:" << endl;
	M.coutReport();
	//M.cFunc = constraintCH34Ctest;
	//M.coutReport();
	/*
	Minimizer M2(50,3,0,xmin,xmax);
	M2.func = testCH34C2;
	M2.coutReport();
	M2.xmin[2]= 2.189;
	M2.coutReport();
	// now for ell + sqrt(delta)*c
	xmin[3]=sqrt(8.0);
	xmax[4]=eM;
	Minimizer M3(50,6,1,xmin,xmax);
	M3.func= testCH34C3;
	M3.cFunc = constraintDelta;
	M3.coutReport();
	M3.xmin[3]=3.2;
	M3.xmax[4]=3.2;
	M3.coutReport();
	*/
	cout << "end-testCH34C" << endl << endl;
}

/**
  * Inequality ch3.5.1 alpha < dih2 or beta < dih2.
  */

double angle(double adj1, double adj2, double op) {
	return acos( (cos(op) - cos(adj1)*cos(adj2))/(sin(adj1)*sin(adj2)) );
}

void testCH351(int,int,double* y,double* ret,void*) {
	double op = arc(y[0],2,2);
	double adj = arc(y[1],getEdgeMax(),2);
	double hyp = arc(y[0],y[1],y[5]);
	double acute = cos(hyp) - cos(op)*cos(adj);
	double a = 0;
	if (acute>0)
		a = angle(hyp,adj,op);
	else
		a = beta(op,hyp);
	*ret = dih(y[1],y[2],y[0],y[4],y[5],y[3]) - a;
			
}

void runCH351() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,2,3.2,2};
	double xmax[6]= {eM,eM,eM,3.2,3.2,sqrt(8.0)};
	Minimizer M(20,6,0,xmin,xmax);
	M.func = testCH351;
	cout << "testCH351:" << endl;
	M.coutReport();
	cout << "end-testCH351" << endl << endl;
}

/**
  * Inequality ch3.5.2 no room for 4 links
  */


void testCH352(int,int,double* y,double* ret,void*) {
	double eM = getEdgeMax();
	double s8 = sqrt(8.0);
	*ret = arc(y[0],y[1],2) + arc(y[0],eM,s8) + arc(eM,eM,2)+arc(eM,y[2],2)
			- arc(y[1],y[2],3.2)- 2.0*arc(y[0],2,2);			
}

void testCH353(int,int,double* y,double* ret,void*) {
	double eM = getEdgeMax();
	double s8 = sqrt(8.0);
	*ret = arc(y[0],y[1],s8) + arc(y[0],eM,2) + arc(eM,eM,2)+arc(eM,y[2],2)
			- arc(y[1],y[2],3.2)- 2.0*arc(y[0],2,2);			
}

void testCH354(int,int,double* y,double* ret,void*) {
	double eM = getEdgeMax();
	double s8 = sqrt(8.0);
	*ret = arc(y[0],eM,s8) + arc(y[0],eM,2) + arc(eM,y[1],2)+arc(eM,y[2],2)
			- arc(y[1],y[2],3.2)- 2.0*arc(y[0],2,2);			
}

void runCH35_234() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2};
	double xmax[6]= {eM,eM,eM};
	Minimizer M(20,3,0,xmin,xmax);
	M.func = testCH352;
	cout << "testCH352:" << endl;
	M.coutReport();
	M.func = testCH353;
	M.coutReport();
	M.func = testCH354;
	M.coutReport();
	cout << "end-testCH352" << endl << endl;
}

/**
  * Inequality ch3.6A, tau_P(U_P) > D(3,k).
  */

double tt(double n) {
	return 2.0*0.017 + (n-4.0)*0.052;
}

double dd(double n,double k) {
	return tt(n+k) -0.017*k;
}

double testCH36Amult0;
double testCH36Amult1;
double testCH36Amult2;

void testCH36A(int,int,double* y,double* ret,void*) {
	double ell0 = L(y[0]);
	double ell1 = L(y[1]);
	double ell2 = L(y[2]);
	double k = 3.0 - (testCH36Amult0+testCH36Amult1+testCH36Amult2)/2.0;
	*ret = taut(y[0],y[1],y[2],y[3],y[4],y[5])
			- testCH36Amult0*ell0/2.0
			- testCH36Amult1*ell1/2.0
			- testCH36Amult2*ell2/2.0
			- dd(3.0,k);
			
}

void runCH36A() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,eM,2,2};
	double xmax[6]= {eM,eM,eM,sqrt(8.0),eM,eM};
	Minimizer M(20,6,0,xmin,xmax);
	M.func = testCH36A;
	cout << "testCH36A:" << endl;
	testCH36Amult0=2.0;
	testCH36Amult1=testCH36Amult2=1.0;
	M.coutReport();
	M.xmax[3]=3.2;
	M.xmin[4]=eM;
	M.xmax[4]=3.2;
	testCH36Amult0=1.0; testCH36Amult2=0.0;
	M.coutReport();
	M.xmin[5]=eM;
	M.xmax[5]=3.2;
	testCH36Amult0=testCH36Amult1=0.0;
	M.coutReport();
	cout << "end-testCH36A" << endl << endl;
}

/**
  * ch3.6B, f''<0 if f'=0.
  * Now for the big one.  We want formulas for the first and second derivatives
  * of the function f.  For the purposes of the proposed inequality we can
  * use x1 derivatives, rather than y1.
  *
  * We also compute
  * (Sqrt[delta] f')' rather than f'', in order to make the derivative better
  * behaved.  Since we are looking at points where f'=0, this doesn't matter.
  * 
  * Let T[f] = Sqrt[delta] D[f,x1]
  * There is a lot to set up for the computation of these second derivatives.
  * The following procedures get used.
  */

// D[L[y],x];
double DL(double x) {
	return (0.3085339999999999 - 0.1199999999999999*sqrt(x))/(2.0*sqrt(x));
}

// T[L[Sqrt[x]]]
double TL(double x, double del) {
	return safesqrt(del)*DL(x);
}

// D[L[y],{x,2}];
double DDL(double x) {
	return -0.0771335/pow(x,1.5); 
}

// T[T[L[Sqrt[x]]]
double TTL(double x, double del1, double del) {
	return (del1/2.0)*DL(x) + del*DDL(x);
}

double B(double x) { // (* phit - A[y/2], edgeMax is hard-wired... *) 
	double y = sqrt(x);
	return -0.4275501864472562 + 0.4092480756249999*y - 0.02083333333333333*pow(y,3.0);
}

// D[B[y],y];
double DB(double x) {
	double y = sqrt(x);
	return (0.4092480756249999 - 0.0625*x)/(2.0*y);
}

double TB(double x,double del) {
	return DB(x)*safesqrt(del);
}

// D[B[y],{x,2}];
double DDB(double x) {
	double y = sqrt(x);
	return -0.4092480756249999/(4.0*x*y) -0.02083333333333333*3.0/(4.0*y);
}

double TTB(double x, double del1, double del) {
	return DB(x)*del1/(2.0) + del*DDB(x);
}

// D[Quoin,a], from Mathematica...
double DaQuoin(double a,double b,double c) {
	double a2 = a*a;
	double b2 = b*b;
	double c2 = c*c;
	 return (-((3.0*a2 - b2 - 2.0*c2)*(-1.0)*sqrt(-(b2 - c2))) + 
     3.0*(a2 - b2)*(a2 - c2)*sqrt(-1.0/(a2 - b2))*
      atan(sqrt((b2 - c2)/(a2 - b2))))/
   (6.0*(a - b)*(a + b)*sqrt(-1.0/(a2 - b2)));
}

// D[Quoin,{a,2}], from Mathematica...
double DaDaQuoin(double a,double b,double c) {
	double a2 = a*a;
	double b2 = b*b;
	double c2 = c*c;
	return    (a*((sqrt(-(b2 - c2))*(3.0*a2 - 4.0*b2 + c2)) + 
       3.0*pow(a2 - b2,2.0)*sqrt(-1.0/(a2 - b2))*
        atan(sqrt((b2 - c2)/(a2 - b2)))))/
   (3.0*pow(a - b,2.0)*pow(a + b,2)*sqrt(-1.0/(a2 - b2)));
}

// D[Quoin,b], from Mathematica...
double DbQuoin(double a,double b,double c) {
	double a2 = a*a;
	double b2 = b*b;
	double c2 = c*c;
	return  (a*(b2 - c2)*sqrt((b2 - c2)/(a2 - b2)))/(3.0*b);
}

// D[D[Quoin,b],a], from Mathematica...
double DaDbQuoin(double a,double b,double c) {
	double a2 = a*a;
	double b2 = b*b;
	double c2 = c*c;
	return (b*(b2 - c2)*sqrt((b2 - c2)/(a2 - b2)))/(3.0*(-a2 + b2));
}

// D[Quoin,{b,2}], from Mathematica...
double DbDbQuoin(double a,double b,double c) {
	double a2 = a*a;
	double b2 = b*b;
	double c2 = c*c;
	return (a*(-1.0)*sqrt(-b + c)*(b + c)*(a2*(2.0*b2 + c2) - 
	b2*(b2 + 2.0*c2)))/
   (3.0*pow(a - b,2)*b2*pow(a + b,2)*sqrt(-(b + c)/(a2 - b2)));
}

// D[y1/2,x1] 
double DaDx1(double x1) {
	return 1.0/(4.0*sqrt(x1));
}

// D[y1/2,{x1,2}]
double DaD2x1(double x1) {
	return  -1.0/(8.0*pow(x1,1.5));
}

// D[eta(x1,x2,x6),x1], from Mathematica
double DbDx1(double x1,double x2,double x3) {
	return   -(sqrt(x2)*(-pow(x1,2) + pow(x2 - x3,2))*sqrt(x3))/
   (2.0*sqrt(x1)*pow(-pow(x1,2) - pow(x2 - x3,2) + 2.0*x1*(x2 + x3),1.5));
}


// D[eta,{x1,2}], from Mathematica.
double DbDDx1(double x1,double x2,double x3) {
	return
   -(sqrt(x2)*sqrt(x3)*(-3.0*pow(x1,4) + 10.0*pow(x1,2)*pow(x2 - x3,2) + pow(x2 - x3,4) - 
        8.0*x1*pow(x2 - x3,2)*(x2 + x3)))/
   (4.0*pow(x1,1.5)*pow(-pow(x1,2) - pow(x2 - x3,2) + 2.0*x1*(x2 + x3),2.5));
}

/**
  * @param a = sqrt(x1)/2,
  * @param b = eta(y1,y3,y5)
  * @param c = eM/2
  * @param del = delta
  */
double TQuoin135(double a,double b,double c,double del,double x1, double x3,double x5) {
	if (b>=c) return 0.0;
	return safesqrt(del)*(DaQuoin(a,b,c)*DaDx1(x1) +
		DbQuoin(a,b,c)*DbDx1(x1,x3,x5));
}

double TQuoin315(double a,double b,double c,double del,double x1, double x3, double x5) {
	if (b>=c) return 0.0;
	return safesqrt(del)*(DbQuoin(a,b,c)*DbDx1(x1,x3,x5));
}

double TTQuoin135(double a,double b,double c,double del, double del1, double x1, double x3,double x5) {
	if (b>=c) return 0.0;
	return (del1/2.0)*(DaQuoin(a,b,c)*DaDx1(x1) +
		DbQuoin(a,b,c)*DbDx1(x1,x3,x5))
		+ del*(DaDaQuoin(a,b,c)*pow(DaDx1(x1),2) + DaQuoin(a,b,c)*DaD2x1(x1) +
			2.0*DaDbQuoin(a,b,c)*DbDx1(x1,x3,x5)*DaDx1(x1)
			+DbDbQuoin(a,b,c)*pow(DbDx1(x1,x3,x5),2)
			+DbQuoin(a,b,c)*DbDDx1(x1,x3,x5)
			);
}

double TTQuoin315(double a, double b, double c, double del, double del1,double x1, double x3, double x5) {
	if (b>=c) return 0.0;
	return (del1/2.0)*(DbQuoin(a,b,c)*DbDx1(x1,x3,x5))
		+del*(DbDbQuoin(a,b,c)*pow(DbDx1(x1,x3,x5),2)+
			DbQuoin(a,b,c)*DbDDx1(x1,x3,x5));
}

// D[uT[x1,x3,x5],x1]
double DuTDx1(double x1,double x3,double x5) {
	return 2.0*(-x1+x3+x5);
}

// D[delta,x1]
double delta1(double x1,double x2,double x3,double x4,double x5,double x6) {
	return  -(x1*x4) + x2*x5 - x3*x5 - x2*x6 + x3*x6 + 
	x4*(-x1 + x2 + x3 - x4 + x5 + x6); 
}

// D[delta,x2]
double delta2(double x1,double x2,double x3,double x4,double x5,double x6) {
	return x1*x4 - x3*x4 - x2*x5 - x1*x6 + x3*x6 + 
	x5*(x1 - x2 + x3 + x4 - x5 + x6);
}

// D[delta,x3]
double delta3(double x1,double x2,double x3,double x4,double x5,double x6) {
	return x1*x4 - x2*x4 - x1*x5 + x2*x5 - x3*x6 + 
	(x1 + x2 - x3 + x4 + x5 - x6)*x6;
}

// D[delta,x4]
double delta4(double x1,double x2,double x3,double x4,double x5,double x6) {
	return -(x2*x3) - x1*x4 + x2*x5 + x3*x6 - 
	x5*x6 + x1*(-x1 + x2 + x3 - x4 + x5 + x6);
}

// D[delta,{x1,2}]
double delta11(double x1,double x2,double x3,double x4,double x5,double x6) {
	return -2.0*x4;
}

// D[delta2,x1]
double delta12(double x1,double x2,double x3,double x4,double x5,double x6) {
	return x4+x5-x6;
}

// D[delta3,x1]
double delta13(double x1,double x2,double x3,double x4,double x5,double x6) {
	return x4-x5+x6;
}

// D[delta4,x1]
double delta14(double x1,double x2,double x3,double x4,double x5,double x6) {
	return  -2.0*x1 + x2 + x3 - 2.0*x4 + x5 + x6;
}

// D[delta14,x1]
double delta114(double x1,double x2,double x3,double x4,double x5,double x6) {
	return -2.0;
}

// D[dih,Dx1] Sqrt[delta]
double Tdih1(double x1,double x2,double x3,double x4,double x5,double x6) {
	double del = delta(x1,x2,x3,x4,x5,x6);
	double de14 = delta14(x1,x2,x3,x4,x5,x6);
	double de4 = delta4(x1,x2,x3,x4,x5,x6);
	double de1 = delta1(x1,x2,x3,x4,x5,x6);
	double u135 = uT(x1,x3,x5);
	double u126 = uT(x1,x2,x6);
	double num = (- 2.0*del*de14*x1 + de4*(del+x1*de1));
	// den = (Sqrt[x1*delta]*u135*u126);
	double dens= (sqrt(x1)*u135*u126);  //(* deprived of Sqrt[delta] in den *)
	return num/dens;
}

// D[D[dih,x1] Sqrt[delta],x1] Sqrt[delta]
double TTdih1(double x1,double x2,double x3,double x4,double x5,double x6) {
	double del = delta(x1,x2,x3,x4,x5,x6);
	double de14 = delta14(x1,x2,x3,x4,x5,x6);
	double de4 = delta4(x1,x2,x3,x4,x5,x6);
	double de1 = delta1(x1,x2,x3,x4,x5,x6);
	double de114 = delta114(x1,x2,x3,x4,x5,x6);
	double de11 = delta11(x1,x2,x3,x4,x5,x6);
	double u135 = uT(x1,x3,x5);
	double u126 = uT(x1,x2,x6);
	double Du135 = DuTDx1(x1,x3,x5);
	double Du126 = DuTDx1(x1,x2,x6);
	double num = (- 2.0*del*de14*x1 + de4*(del+x1*de1));
	// den = (Sqrt[x1*delta]*u135*u126);
	double dens= (sqrt(x1)*u135*u126);  //(* deprived of Sqrt[delta] in den *)
	double Dnum = -2.0*de1*de14*x1 - 2.0*del*de114*x1 - 2.0*del*de14 +
		de14*(del+x1*de1) + de4*(de1+de1+x1*de11);
	double Ddens = 0.5/sqrt(x1)*u135*u126+
		sqrt(x1)*(Du135*u126+u135*Du126);
	return  safesqrt(del)*(Dnum*dens - num*Ddens)/pow(dens,2);
}


// D[dih2,x1] Sqrt[delta]
double  Tdih2(double x1,double x2,double x3,double x4,double x5,double x6) {
	double de3 = delta3(x1,x2,x3,x4,x5,x6);
	double y2 = sqrt(x2);
	double u126 = uT(x1,x2,x6);
	return -de3*y2/u126;
}

// D[D[dih2,x1] Sqrt[delta],x1] Sqrt[delta]
double  TTdih2(double x1,double x2,double x3,double x4,double x5,double x6) {
	double de3 = delta3(x1,x2,x3,x4,x5,x6);
	double de = delta(x1,x2,x3,x4,x5,x6);
	double y2 = sqrt(x2);
	double u126 = uT(x1,x2,x6);
	double de13 = delta13(x1,x2,x3,x4,x5,x6);
	double Du126 = DuTDx1(x1,x2,x6);
	return safesqrt(de)*(-de13*y2*u126 + de3*y2*Du126)/pow(u126,2);
}

// D[dih3,x1] Sqrt[delta]
double  Tdih3(double x1,double x2,double x3,double x4,double x5,double x6) {
	double de2 = delta2(x1,x2,x3,x4,x5,x6);
	double y3 = sqrt(x3);
	double u135 = uT(x1,x3,x5);
	return -de2*y3/u135;
}

// D[D[dih3,x1] Sqrt[delta],x1] Sqrt[delta]
double  TTdih3(double x1,double x2,double x3,double x4,double x5,double x6) {
	double de2 = delta2(x1,x2,x3,x4,x5,x6);
	double de = delta(x1,x2,x3,x4,x5,x6);
	double y3 = sqrt(x3);
	double u135 = uT(x1,x3,x5);
	double de12 = delta12(x1,x2,x3,x4,x5,x6);
	double Du135 = DuTDx1(x1,x3,x5);
	return safesqrt(de)*(-de12*y3*u135 + de2*y3*Du135)/pow(u135,2);
}

double TQuoinTerm(double x1,double x2,double x3,double x4,double x5,double x6) {
	double y1 = sqrt(x1);
	double y3 = sqrt(x3);
	double y2 = sqrt(x2);
	double y5 = sqrt(x5);
	double y6 = sqrt(x6);
	double a = y1/2.0;
	double b = eta(y1,y3,y5);
	double b126 = eta(y1,y2,y6);
	double c = getTrunc();
	double de = delta(x1,x2,x3,x4,x5,x6);
	return TQuoin135(a,b,c,de,x1,x3,x5)+
		TQuoin315(y3/2.0,b,c,de,x1,x3,x5)+
		TQuoin135(a,b126,c,de,x1,x2,x6)+
		TQuoin315(y2/2.0,b126,c,de,x1,x2,x6);
}

double TTQuoinTerm(double x1,double x2,double x3,double x4,double x5,double x6) {
	double y1 = sqrt(x1);
	double y3 = sqrt(x3);
	double y2 = sqrt(x2);
	double y5 = sqrt(x5);
	double y6 = sqrt(x6);
	double a = y1/2.0;
	double b = eta(y1,y3,y5);
	double b126 = eta(y1,y2,y6);
	double c = getTrunc();
	double de = delta(x1,x2,x3,x4,x5,x6);
	double de1 = delta1(x1,x2,x3,x4,x5,x6);
	return TTQuoin135(a,b,c,de,de1,x1,x3,x5)
			+TTQuoin315(y3/2.0,b,c,de,de1,x1,x3,x5)
			+TTQuoin135(a,b126,c,de,de1,x1,x2,x6)
			+TTQuoin315(y2/2.0,b126,c,de,de1,x1,x2,x6);
}

double TDihTerm(double x1,double x2,double x3,double x4,double x5,double x6) {
	double de = delta(x1,x2,x3,x4,x5,x6);
	double y1 = sqrt(x1);
	double dih1 = dih(y1,sqrt(x2),sqrt(x3),sqrt(x4),sqrt(x5),sqrt(x6));
	return TB(x1,de)*dih1 
		    + B(x1)*Tdih1(x1,x2,x3,x4,x5,x6)
			+ B(x2)*Tdih2(x1,x2,x3,x4,x5,x6)
			+ B(x3)*Tdih3(x1,x2,x3,x4,x5,x6);
}

double TTDihTerm(double x1,double x2,double x3,double x4,double x5,double x6) {
	double de = delta(x1,x2,x3,x4,x5,x6);
	double de1 = delta1(x1,x2,x3,x4,x5,x6);
	double y1 = sqrt(x1);
	double dih1 = dih(y1,sqrt(x2),sqrt(x3),sqrt(x4),sqrt(x5),sqrt(x6));
	return TTB(x1,de1,de)*dih1 + 2.0*TB(x1,de)*Tdih1(x1,x2,x3,x4,x5,x6)+
			B(x1)*TTdih1(x1,x2,x3,x4,x5,x6)
			+ B(x2)*TTdih2(x1,x2,x3,x4,x5,x6)
			+ B(x3)*TTdih3(x1,x2,x3,x4,x5,x6);
}

double TLTerm(double x1,double x2,double x3,double x4,double x5,double x6) {
	double de = delta(x1,x2,x3,x4,x5,x6);
	return -(TL(x1,de))/2.0;

}

double TTLTerm(double x1,double x2,double x3,double x4,double x5,double x6) {
	double de = delta(x1,x2,x3,x4,x5,x6);
	double de1 =delta1(x1,x2,x3,x4,x5,x6);
	return -(TTL(x1,de1,de))/2.0;

}

double TTfTerm(double x1,double x2,double x3,double x4,double x5,double x6,double mult) {
	return TTQuoinTerm(x1,x2,x3,x4,x5,x6)+TTDihTerm(x1,x2,x3,x4,x5,x6)+
		TTLTerm(x1,x2,x3,x4,x5,x6)*mult;
}

double TfTerm(double x1,double x2,double x3,double x4,double x5,double x6,double mult) {
	return TQuoinTerm(x1,x2,x3,x4,x5,x6)+TDihTerm(x1,x2,x3,x4,x5,x6)+
		TLTerm(x1,x2,x3,x4,x5,x6)*mult;
}

double testCH36Bmult;
void testCH36B(int,int,double* y,double* ret,void*) {
	double x1 = y[0]*y[0];
	double x2 = y[1]*y[1];
	double x3 = y[2]*y[2];
	double x4 = y[3]*y[3];
	double x5 = y[4]*y[4];
	double x6 = y[5]*y[5];
	*ret = -TTfTerm(x1,x2,x3,x4,x5,x6,testCH36Bmult)
		+10.0*pow(TfTerm(x1,x2,x3,x4,x5,x6,testCH36Bmult),2); // penalty fn.
}

void constraintCH36B(int,int,double* y,double* ret,void*) {
		*ret= -delta(y[0]*y[0],y[1]*y[1],y[2]*y[2],y[3]*y[3],y[4]*y[4],y[5]*y[5]);
}

void testCodeCH36B() {
	double x1 = 4.1;
	double x2 = 4.2;
	double x3 = 4.3;
	double x4 = 4.4;
	double x5 = 4.5;
	double x6 = 4.6;
	double y1 = sqrt(x1);
	double y2 = sqrt(x2);
	double y3 = sqrt(x3);
	double y4 = sqrt(x4);
	double y5 = sqrt(x5);
	double y6 = sqrt(x6);
	double a = y1/2.0;
	double b126 = eta(y1,y2,y6);
	double b = b126;
	double b135 = eta(y1,y3,y5);
	double c = getTrunc();
	double de = delta(x1,x2,x3,x4,x5,x6);
	double de1 = delta1(x1,x2,x3,x4,x5,x6);
	/*
	cout << "TL(x1)= " << TL(x1,de) << endl;
	cout << "TTL(x1)= " << TTL(x1,de1,de) << endl;
	cout << "B(x1)= " << B(x1) << endl;
	cout << "TB(x1)= " << TB(x1,de) << endl;
	cout << "TTB(x1)= " << TTB(x1,de1,de) << endl;
	cout << "DaQuoin= " << DaQuoin(a,b,c) << endl;
	cout << "DbQuoin= " << DbQuoin(a,b,c) << endl;
	cout << "DaDaQuoin= " << DaDaQuoin(a,b,c) << endl;
	cout << "DaDbQuoin= " << DaDbQuoin(a,b,c) << endl;
	cout << "DbDbQuoin= " << DbDbQuoin(a,b,c) << endl;
	cout << "DaDx1 = " << DaDx1(x1) << endl;
	cout << "DaD2x1= " << DaD2x1(x1) << endl;
	cout << "DbDx1= " << DbDx1(x1,x2,x6) << endl;
	cout << "DbDDx1=" << DbDDx1(x1,x2,x6) << endl;
	cout << "DuTDx1=" << DuTDx1(x1,x2,x6) << endl;
	cout << "TQuoin135=" << TQuoin135(a,b135,c,de,x1,x3,x5) << endl;
	cout << "TQuoin315=" << TQuoin315(y3/2.0,b135,c,de,x1,x3,x5)<< endl;
	cout << "TTQuoin135=" << TTQuoin135(a,b135,c,de,de1,x1,x3,x5)<< endl;
	cout << "TTQuoin315=" << TTQuoin315(y3/2.0,b135,c,de,de1,x1,x3,x5)<< endl;
	cout << "delta1=" << delta1(x1,x2,x3,x4,x5,x6)<<endl;
	cout << "delta2=" << delta2(x1,x2,x3,x4,x5,x6)<<endl;
	cout << "delta3=" << delta3(x1,x2,x3,x4,x5,x6)<<endl;
	cout << "delta4=" << delta4(x1,x2,x3,x4,x5,x6)<<endl;
	cout << "delta11=" << delta11(x1,x2,x3,x4,x5,x6)<<endl;
	cout << "delta12=" << delta12(x1,x2,x3,x4,x5,x6)<<endl;
	cout << "delta13=" << delta13(x1,x2,x3,x4,x5,x6)<<endl;
	cout << "delta14=" << delta14(x1,x2,x3,x4,x5,x6)<<endl;
	cout << "delta114=" << delta114(x1,x2,x3,x4,x5,x6)<<endl;
	cout << "Tdih1=" << Tdih1(x1,x2,x3,x4,x5,x6) << endl;
	cout << "TTdih1=" << TTdih1(x1,x2,x3,x4,x5,x6) << endl;
	cout << "Tdih2=" << Tdih2(x1,x2,x3,x4,x5,x6) << endl;
	cout << "TTdih2=" << TTdih2(x1,x2,x3,x4,x5,x6) << endl;
	cout << "Tdih3=" << Tdih3(x1,x2,x3,x4,x5,x6) << endl;
	cout << "TTdih3=" << TTdih3(x1,x2,x3,x4,x5,x6) << endl;
	cout << "TTQuoinTerm=" << TTQuoinTerm(x1,x2,x3,x4,x5,x6) << endl;
	cout << "TTDihTerm=" << TTDihTerm(x1,x2,x3,x4,x5,x6) << endl;
	cout << "TTLTerm=" << TTLTerm(x1,x2,x3,x4,x5,x6) << endl;
	*/
	cout << "TQuoinTerm=" << TQuoinTerm(x1,x2,x3,x4,x5,x6) << endl;
	cout << "TDihTerm=" << TDihTerm(x1,x2,x3,x4,x5,x6) << endl;
	cout << "TLTerm=" << TLTerm(x1,x2,x3,x4,x5,x6) << endl;

}

void runCH36B() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,2,2,2};
	double xmax[6]= {eM,eM,eM,4,2,2};
	Minimizer M(5,6,1,xmin,xmax);
	M.func = testCH36B;
	M.cFunc = constraintCH36B;
	cout << "testCH36B:" << endl;
	testCH36Bmult=2.0;
	M.coutReport();
	// other cases (2,eM), (2,sqrt(8)), (eM,eM) (eM,sqrt(8)), (s8,s8)
	testCH36Bmult=1.0;
	M.xmin[5]=eM;
	M.xmax[5]=eM;
	M.xmax[3]=2.0*eM;
	M.coutReport();
	M.xmin[5]=sqrt(8.);
	M.xmax[5]=sqrt(8.);
	M.coutReport();
	testCH36Bmult=0.0;
	M.xmin[5]=eM;
	M.xmax[5]=eM;
	M.xmin[4]=eM;
	M.xmax[4]=eM;
	M.coutReport();
	M.xmin[5]=sqrt(8.);
	M.xmax[5]=sqrt(8.);
	M.coutReport();
	M.xmin[4]=sqrt(8.);
	M.xmax[4]=sqrt(8.);
	M.coutReport();
	cout << "end-testCH36B" << endl << endl;
}

/**
  * Inequality ch3.6C, tau_P(quad) > D(4,1);
  */
void testCH36C(int,int,double* y,double* ret,void*) {
	*ret = taut(y[0],y[1],y[2],y[3],y[4],y[5])
		+taut(2,y[1],y[2],y[3],2,2)
		-L(y[0])/2.0 - L(y[1])/2.0
		- dd(4,1);
}


void runCH36C() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,sqrt(8.),eM,2};
	double xmax[6]= {eM,eM,2,3.2,3.2,2};
	Minimizer M(20,6,0,xmin,xmax);
	M.func = testCH36C;
	cout << "testCH36C:" << endl;
	M.coutReport();
	cout << "end-testCH36C" << endl << endl;
}

/**
  * Inequality ch3.7.6, tau_P(special) > D(3,1), edge at diag = eM.
  */
void testCH376(int,int,double* y,double* ret,void*) {
	*ret = taut(y[0],y[1],y[2],y[3],y[4],y[5])
			-L(y[0]) - L(y[1])/2.0 - L(y[2])/2.0
			-dd(3,1);
}

void constraintCH376(int,int,double* y,double* ret,void*) {
		*ret= -y[1]-y[5]+getEdgeMax()+2.0;
}

void runCH376() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,sqrt(8.),2,2};
	double xmax[6]= {eM,eM,eM,3.2,eM,eM};
	cout << "D(3,1)=" << dd(3,1) << endl;
	Minimizer M(20,6,1,xmin,xmax);
	M.func = testCH376;
	M.cFunc = constraintCH376;
	cout << "testCH376:" << endl;
	M.coutReport();
	cout << "end-testCH376" << endl << endl;
}

/**
  * Inequality ch3.7.7, tau_P(special) > XX.
  */
void testCH377(int,int,double* y,double* ret,void*) {
	*ret = taut(y[0],y[1],y[2],y[3],y[4],y[5])
			-L(y[0]) - L(y[1])/2.0 - L(y[2])/2.0
			-0.0;
}

void constraintCH377(int,int,double* y,double* ret,void*) {
		*ret= -delta(y[0]*y[0],y[1]*y[1],y[2]*y[2],y[3]*y[3],y[4]*y[4],y[5]*y[5]);
}

void runCH377() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,sqrt(8.),2,2};
	double xmax[6]= {2,2,2,3.2,2,2};
	Minimizer M(20,6,1,xmin,xmax);
	M.func = testCH377;
	M.cFunc = constraintCH377;
	cout << "testCH377:" << endl;
	M.coutReport();
	M.xmin[0]=eM;
	M.xmax[0]=eM;
	M.xmax[3]=3.0744;
	M.coutReport();
	cout << "end-testCH377" << endl << endl;
}

/**
  * Inequality ch3.7C, a special quad case. (1 d)
  */
void testCH37C(int,int,double* y,double* ret,void*) {
	*ret = taut(y[0],y[1],y[2],y[3],y[4],y[5])*2.0 - dd(4,4);
}

void runCH37C() {
	double eM = getEdgeMax();
	double xmin[6]= {eM,eM,eM,3.2,eM,eM};
	double xmax[6]= {eM,eM,eM,eM*sqrt(2.0),eM,eM};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testCH37C;
	cout << "testCH37C:" << endl;
	M.coutReport();
	cout << "end-testCH37C" << endl << endl;
}

/*****
  *
  * CHAPTER 4 stuff
  */
/**
  * Inequality ch4.3B5, dih3 > 1.47 > beta_psi,
  */
void testCH43B5a(int,int,double* y,double* ret,void*) {
	*ret = dih(y[2],y[0],y[1],y[5],y[3],y[4]) - 1.47;
}

double lambda4X = 1.42;
void testCH43B5b(int,int,double* y,double* ret,void*) {
	*ret = 1.47 - beta(arc(2.0,getTrunc(),lambda4X),arc(y[0],y[1],y[2]));
}

void runCH43B5a() {
	double eM = getEdgeMax();
	double xmin[6]= {2,eM,eM,3.2,2,3.2};
	double xmax[6]= {sqrt(6.),eM,eM,3.2,2,3.2};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testCH43B5a;
	cout << "testCH43B5a:" << endl;
	M.coutReport();
	cout << "end-test43B5a" << endl << endl;
}

void runCH43B5b() {
	double eM = getEdgeMax();
	double xmin[3]= {2,eM,2};
	double xmax[3]= {sqrt(6.),eM,2};
	Minimizer M(50,3,0,xmin,xmax);
	M.func = testCH43B5b;
	cout << "testCH43B5b:" << endl;
	M.coutReport();
	cout << "end-test43B5b" << endl << endl;
}

void runCH43B5() {
	runCH43B5a();
	runCH43B5b();
}

/**
  * Inequality ch4.3B6, dih3 > 0.7 > beta_psi,
  */
void testCH43B6a(int,int,double* y,double* ret,void*) {
	*ret = dih(y[2],y[0],y[1],y[5],y[3],y[4]) - 0.7;
}

void testCH43B6b(int,int,double* y,double* ret,void*) {
	*ret = 0.7 - beta(arc(sqrt(6.0),getTrunc(),lambda4X),arc(y[0],y[1],y[2]));
}

void runCH43B6a() {
	double eM = getEdgeMax();
	double xmin[6]= {sqrt(6.0),eM,eM,3.2,2,3.2};
	double xmax[6]= {eM,eM,eM,3.2,2,3.2};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testCH43B6a;
	cout << "testCH43B6a:" << endl;
	M.coutReport();
	cout << "end-test43B6a" << endl << endl;
}

void runCH43B6b() {
	double eM = getEdgeMax();
	double xmin[3]= {sqrt(6.0),eM,2};
	double xmax[3]= {eM,eM,2};
	Minimizer M(50,3,0,xmin,xmax);
	M.func = testCH43B6b;
	cout << "testCH43B6b:" << endl;
	M.coutReport();
	cout << "end-test43B6b" << endl << endl;
}

void runCH43B6() {
	runCH43B6a();
	runCH43B6b();
}

/**
  * Inequality ch4.3C1, dih2 >  beta_psi,
  */
void testCH43C1(int,int,double* y,double* ret,void*) {
	*ret = dih(y[2],y[0],y[1],y[5],y[3],y[4])  
		- beta(arc(y[0],getTrunc(),1.6),arc(y[0],y[2],y[4]));
}

void runCH43C1() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,2,eM,eM};
	double xmax[6]= {eM,eM,eM,3.2,eM,eM};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testCH43C1;
	cout << "testCH43C1:" << endl;
	M.coutReport();
	cout << "end-test43C1" << endl << endl;
}

/**
  * Inequality ch4.3C2, dih2 >  beta_psi,
  */
void testCH43C2(int,int,double* y,double* ret,void*) {
	*ret = dih(y[2],y[0],y[1],y[5],y[3],y[4])  
		- beta(arc(y[0],getTrunc(),1.6),arc(y[0],y[2],y[4]));
}

void runCH43C2() {
	double eM = getEdgeMax();
	double xmin[6]= {2.2,2,2,     2,2,3.2};
	double xmax[6]= {eM,eM,eM, 3.2,2,3.2};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testCH43C2;
	cout << "testCH43C2:" << endl;
	M.coutReport();
	cout << "end-test43C2" << endl << endl;
}

/**
  * Inequality ch4.3C5, dih2 >  dihR,
  */
double dihR(double a, double b, double c) {
	if (b>c || a>b) return 0.0; 
	return dih(a,b,c,sqrt(c*c-b*b),sqrt(c*c-a*a),sqrt(b*b-a*a));
}

double solR(double a, double b, double c) {
	if (b>c || a>b) return 0.0;
	return solid(a,b,c,sqrt(c*c-b*b),sqrt(c*c-a*a),sqrt(b*b-a*a));
}

void testCH43C5(int,int,double* y,double* ret,void*) {
	double eta126 = eta(y[0],y[1],y[5]);
	double cpsi = cos(arc(y[0],getTrunc(),1.6));
	*ret = dih(y[1],y[2],y[0],y[4],y[5],y[3])  
		- dihR(y[1]/2.0,eta126,y[0]/(2.0*cpsi));
}

void runCH43C5() {
	double eM = getEdgeMax();
	double xmin[6]= {2.2,2,eM,     3.2,3.2,2};
	double xmax[6]= {2.2,eM,eM, 3.2,3.2,2};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testCH43C5;
	cout << "testCH43C5:" << endl;
	M.coutReport();
	cout << "end-test43C5" << endl << endl;
}

/**
  * Inequality ch4.7B, chiMinus > 0, mu=1.
  */
double solp(double cospsi,double y1,double eta126) {
	double a = y1/2.0;
	double b = eta126;
	double c = y1/(2.0*cospsi);
	return dihR(a,b,c)*(1.0-cospsi) - solR(a,b,c); 
}

double getPhit() {
	return phi0() - 1.0/(3.0*getDtet());
}

double chiMinus(double y1, double y2, double y3, double y4, double y5, double y6,
			  double lambda,
			  double mu) {
	double psi = arc(y1,getTrunc(),lambda);
	double eta126 = eta(y1,y2,y6);
	double eta135 = eta(y1,y3,y5);
	double cospsi = cos(psi);
	double phit = getPhit();
	double dihS = dih(y1,y2,y3,y4,y5,y6);
	double sol126 = solp(cospsi,y1,eta126);
	double sol135 = solp(cospsi,y1,eta135);
	return dihS*(1.0-cospsi)*phit
			-sol126*phit - sol135*phit
			-A(y1/2.0)*dihS 
			+ quoin(y1/2.0,eta126,getTrunc())
			+ quoin(y1/2.0,eta135,getTrunc())
			- mu*L(y1)/2.0;
}

// doesn't depend on y4...
double chiPi(double y1, double eta126, double eta135,
			 double lambda) {
	double psi = arc(y1,getTrunc(),lambda);
	double cospsi = cos(psi);
	double phit = getPhit();
	double dihS = 4.0*atan(1.0); //pi
	double sol126 = solp(cospsi,y1,eta126);
	double sol135 = solp(cospsi,y1,eta135);
	return dihS*(1.0-cospsi)*phit
			-sol126*phit - sol135*phit
			-A(y1/2.0)*dihS;
}

// doesn't depend on y4...
double chiPi(double y1, double y2, double y3, double y4, double y5, double y6,
			 double lambda) {
	double eta126 = eta(y1,y2,y6);
	double eta135 = eta(y1,y3,y5);
	return chiPi(y1,eta126,eta135,lambda);
}

double testCH47Blambda;
void testCH47B(int,int,double* y,double* ret,void*) {
	*ret = chiMinus(y[0],y[1],y[2],y[3],y[4],y[5],testCH47Blambda,1.0);
}

void runCH47B() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,     sqrt(8.0),2,eM};
	double xmax[6]= {2.42,eM,eM,  sqrt(8.0),eM,3.2};
	testCH47Blambda=1.3;
	//cout << "phit " << getPhit() << endl;
	//cout << "solp " << solp(0.7,2.0,1.3) << endl;
	//cout << "L = " << L(2.1) << endl;
	//cout << "chiMinus = " << chiMinus(2.01,2.02,2.03,2.04,2.05,2.06,1.6,2.1) << endl;
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testCH47B;
	cout << "testCH47B:" << endl;
	M.coutReport();
	M.xmax[0]=eM;
	testCH47Blambda=lambda4X;
	M.coutReport();
	cout << "end-test47B" << endl << endl;
}

/**
  * Inequality ch4.7C, chiMinus > 0, mu=2.
  */
double testCH47Clambda;
void testCH47C(int,int,double* y,double* ret,void*) {
	*ret = chiMinus(y[0],y[1],y[2],y[3],y[4],y[5],testCH47Clambda,2.0);
}

void runCH47C() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,     sqrt(8.0),2,2};
	double xmax[6]= {2.42,eM,eM,  sqrt(8.0),eM,eM};
	testCH47Clambda=1.3;
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testCH47C;
	cout << "testCH47C:" << endl;
	M.coutReport();
	M.xmax[0]=eM;
	testCH47Clambda=1.45;
	M.coutReport();
	cout << "end-test47C" << endl << endl;
}

/**
  * Inequality ch4.8B, tau0(C) > 0.1.
  */

void testCH48B(int,int,double* y,double* ret,void*) {
	*ret = chiPi(y[0],y[1],y[2],y[3],y[4],y[5],1.6)-0.1;
}

void runCH48B() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,     3.2,3.07,3.07};
	double xmax[6]= {eM,eM,eM,  3.2,3.2,3.2};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testCH48B;
	cout << "testCH48B:" << endl;
	M.coutReport();
	cout << "end-test48B" << endl << endl;
}

/**
  * Inequality ch4.8C, tau0(C) > 0.2.
  */

void testCH48C(int,int,double* y,double* ret,void*) {
	*ret = chiPi(y[0],y[1],y[2],y[3],y[4],y[5],1.8)-0.2;
}

void runCH48C() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,     3.2,3.07,3.07};
	double xmax[6]= {2.2,eM,eM,  3.2,3.2,3.2};
	Minimizer M(50,6,0,xmin,xmax);
	M.func = testCH48C;
	cout << "testCH48C:" << endl;
	M.coutReport();
	M.xmax[0]=eM;
	M.xmin[4]=3.2;
	M.xmin[5]=3.2;
	M.coutReport();
	cout << "end-test48C" << endl << endl;
}

/**
  * Inequality ch4.8D, double system > sqL
  */

double fnCH48D(double y1,double y2) {
	double lambda = 1.8;
	double ell = 3.2;
	double a1 = dih(y1,getTrunc(),y2,lambda,ell,lambda);
	double a2 = dih(y2,getTrunc(),y1,lambda,ell,lambda);
	double s = solid(y2,getTrunc(),y1,lambda,ell,lambda);
	double pi = 4.0*atan(1.0);
	return 2.0*getPhit()*s - 2.0*a1*A(y1/2.0) - 2.0*a2*A(y2/2.0)
		+ (pi-2.0*a1)*chiPi(y1,3.07/2,3.07/2,lambda)/pi
		+ (pi-2.0*a2)*chiPi(y2,3.07/2,3.07/2,lambda)/pi;
}

void testCH48D(int,int,double* y,double* ret,void*) {
	double sqL = sqrt(32.0) - 16.0*atan(1.0)/(3.0*getDtet());
	*ret = fnCH48D(y[0],y[1])-sqL;
}

void runCH48D() {
	double eM = getEdgeMax();
	double xmin[2]= {2,2};
	double xmax[2]= {eM,eM};
	Minimizer M(50,2,0,xmin,xmax);
	M.func = testCH48D;
	cout << "testCH48D:" << endl;
	M.coutReport();
	cout << "end-test48D" << endl << endl;
}

/*****
  * CHAPTER 6 stuff
  */

/**
  * Inequality ch6.2E, local optimality of tetrahedron
  */
void testCH62E(int,int,double* y,double* ret,void*) {
	*ret = volan(y[0],y[1],y[2],y[3],y[4],y[5])-volan(2,2,2,2,2,2)
		+ 0.062*(y[3]+y[4]+y[5]-6.0)
		- 0.0686*(y[0]+y[1]+y[2]-6.0)
		-0.72*(solid(y[0],y[1],y[2],y[3],y[4],y[5])-solid(2,2,2,2,2,2));
}

void runCH62E() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,2,2,2};
	double xmax[6]= {2.1,2.1,2.1,2.1,2.1,2.1};
	Minimizer M(20,6,0,xmin,xmax);
	M.func = testCH62E;
	cout << "testCH62E:" << endl;
	M.coutReport();
	cout << "end-testCH62E" << endl << endl;
}

/**
  * Inequality ch6.2F, local optimality of flat quarter, analytic case
  */
void testCH62F1(int,int,double* y,double* ret,void*) {
	*ret = volan(y[0],y[1],y[2],y[3],y[4],y[5])-volan(2,2,2,sqrt(8.0),2,2)
		-0.72*(solid(y[0],y[1],y[2],y[3],y[4],y[5])-solid(2,2,2,sqrt(8.0),2,2))
		+ 0.05648*(y[4]+y[5]-4.0) // was 0.06062
		+ 0.003*(sqrt(8.)-y[3])
		- 0.0686*(y[0]-2.0)
		- 0.0343*(y[1]+y[2]-4.0)
		;
}

/**
  * Inequality ch6.2F, local optimality of flat quarter, sqrt(2)-trunc case
  */
void testCH62F2(int,int,double* y,double* ret,void*) {
	*ret = volq(y[0],y[1],y[2],y[3],y[4],y[5],sqrt(2.0))
		-volq(2,2,2,sqrt(8.0),2,2,sqrt(2.0))
		-0.72*(solid(y[0],y[1],y[2],y[3],y[4],y[5])-solid(2,2,2,sqrt(8.0),2,2))
		+ 0.05648*(y[4]+y[5]-4.0) // was 0.06062
		+ 0.003*(sqrt(8.)-y[3])
		- 0.0686*(y[0]-2.0)
		- 0.0343*(y[1]+y[2]-4.0)
		;
}

void constraintCH62F2(int,int,double* y,double* ret,void*) {
	*ret = -eta(y[3],y[4],y[5])+sqrt(2.0);
}

void constraintCH62F3(int,int,double* y,double* ret,void*) {
	*ret = -eta(y[1],y[2],y[3])+sqrt(2.0);
}

void runCH62F() {
	double eM = getEdgeMax();
	double xmin[6]= {2,2,2,eM,2,2};
	double xmax[6]= {2.1,2.1,2.1,sqrt(8.0),2.1,2.1};
	cout << "volt = " << volt(2.04,2.08,2.12,2.16,2.2,2.24) << " " << volq(2.04,2.08,2.12,2.16,2.2,2.24,getTrunc()) << endl;
	Minimizer M(20,6,0,xmin,xmax);
	M.func = testCH62F1;
	cout << "testCH62F:" << endl;
	M.coutReport();
	Minimizer M2(20,6,1,xmin,xmax);
	M2.func = testCH62F2;
	M2.cFunc = constraintCH62F2;
	M2.coutReport();
	M2.cFunc = constraintCH62F3;
	M2.coutReport();
	cout << "end-testCH62F" << endl << endl;
}

/**
  * Inequality ch6.2G, local optimality of upright quarter, analytic case
  */
double testCH62Gfactor; // global either 0.07 or 0.09

void testCH62G1(int,int,double* y,double* ret,void*) {
	*ret = volan(y[0],y[1],y[2],y[3],y[4],y[5])
		-volan(sqrt(8.0),2,2,2,2,2)
		-0.72*(solid(y[0],y[1],y[2],y[3],y[4],y[5])-solid(sqrt(8.0),2,2,2,2,2))
		+ 0.062*(y[3]-2.0)
		- 0.0343*(y[1]+y[2]-4.0)
		- M(y[0])/2.0
		- testCH62Gfactor*(dih(y[0],y[1],y[2],y[3],y[4],y[5])-2.0*atan(1.0))
		;
}


void testCH62G2(int,int,double* y,double* ret,void*) {
	*ret = volq(y[0],y[1],y[2],y[3],y[4],y[5],sqrt(2.0))
		-volq(sqrt(8.0),2,2,2,2,2,sqrt(2.0))
		-0.72*(solid(y[0],y[1],y[2],y[3],y[4],y[5])-solid(sqrt(8.0),2,2,2,2,2))
		+ 0.062*(y[3]-2.0)
		- 0.0343*(y[1]+y[2]-4.0)
		- M(y[0])/2.0
		- testCH62Gfactor*(dih(y[0],y[1],y[2],y[3],y[4],y[5])-2.0*atan(1.0))
		;
}

void constraintCH62G2(int,int,double* y,double* ret,void*) {
	*ret = -eta(y[0],y[1],y[5])+sqrt(2.0);
}

void constraintCH62G3(int,int,double* y,double* ret,void*) {
	*ret = -eta(y[0],y[2],y[4])+sqrt(2.0);
}

void runCH62G() {
	double eM = getEdgeMax();
	double xmin[6]= {2.7,2,2,2,2,2};
	double xmax[6]= {sqrt(8.0),2.1,2.1,2.1,2.41,2.41};
	Minimizer M(50,6,0,xmin,xmax);
	testCH62Gfactor = 0.07;
	M.func = testCH62G1;
	cout << "testCH62G:" << endl;
	M.coutReport();
	Minimizer M2(20,6,1,xmin,xmax);
	M2.func = testCH62G2;
	M2.cFunc = constraintCH62G2;
	M2.coutReport();
	cout << "now test between 2.644 and 2.7...." << endl;
	M.xmin[0]=2.644;  
	M.xmax[0]=2.7;
	testCH62Gfactor = 0.09;
	M.coutReport();
	cout << "end-testCH62G" << endl << endl;
}

/**
  * Inequality ch6.2H, vol_R(V(sqrt(2))) ... uniquely minimized for regular figure.
  * variable ordering as in crossdiag of kepfn.h
  */
void testCH62H(int,int,double* y,double* ret,void*) {
	*ret = (volq(y[0],y[1],y[2],y[3],y[4],y[5],sqrt(2.0)) +
		   volq(y[6],y[1],y[2],y[3],y[7],y[8],sqrt(2.0)))
		   -2.0*volq(2,2,2,sqrt(8.0),2,2,sqrt(2.0))
		   -0.72*(solid(y[0],y[1],y[2],y[3],y[4],y[5]) +
		   solid(y[6],y[1],y[2],y[3],y[7],y[8])
		   -2.0*solid(2,2,2,sqrt(8.0),2,2)
		   )
		   +0.06062*(y[4]+y[5]+y[7]+y[8]-8.0)
		   -0.0686*(y[0]+y[1]+y[2]+y[6]-8.0);
}

void constraintCH62H(int,int,double* y,double* ret,void*) {
	*ret = -crossdiag(y) + safesqrt(8.0);
}

void runCH62H() {
	double eM = getEdgeMax();
	double xmin[9]= {2,2,2,sqrt(8.0),2,2,  2,2,2};
	double xmax[9]= {2.1,2.1,2.1,3.331,2.1,2.1,  2.1,2.1,2.1};
	Minimizer M(20,9,1,xmin,xmax);
	M.func = testCH62H;
	M.cFunc = constraintCH62H;
	cout << "testCH62H:" << endl;
	M.coutReport();
	cout << "crossdiag = " << crossdiag(M.x) << endl;
	cout << "end-testCH62H" << endl << endl;
}

/***
  *  APPENDIX (LINEAR PROGRAMMING) INEQUALITIES
  */

/**
  * apLP1.C, 
  */
void testAPLP1C(int,int,double* y,double* ret,void*) {
	*ret = volan(y[0],y[1],y[2],y[3],y[4],y[5])-
		volt(y[0],y[1],y[2],y[3],y[4],y[5])+0.000507;
}

void runAPLP1C() {
	double xmin[6]= {2,2,2,2,2,2};
	double eM = getEdgeMax();
	double xmax[6]= {eM,eM,eM,eM,eM,eM};
	Minimizer M(40,6,0,xmin,xmax);
	M.func = testAPLP1C;
	cout << "testAPLP1C:" << endl;
	M.coutReport();
	cout << "end-testAPLP1C" << endl << endl;
}

/**
  * apLP1.D, lower linear bound on quoin
  */
void testAPLP1D(int,int,double* y,double* ret,void*) {
	*ret = qy(y[0],y[1],y[2]) - 0.0036 + 0.01*(y[0]-2.0) 
		+0.0135*(y[1]+y[2]-4.0);
}

void runAPLP1D() {
	double xmin[3]= {2,2,2};
	double eM = getEdgeMax();
	double xmax[3]= {eM,eM,eM};
	Minimizer M(40,3,0,xmin,xmax);
	M.func = testAPLP1D;
	cout << "testAPLP1D:" << endl;
	M.coutReport();
	cout << "end-testAPLP1D" << endl << endl;
}

/**
  * apLP1.F, lower linear bound on quoin
  */
double qsym(double y1, double y2, double y3) {
	return qy(y1,y2,y3) + qy(y2,y1,y3);
}

void testAPLP1F(int,int,double* y,double* ret,void*) {
	*ret = qsym(y[0],y[1],y[2]) +L(y[2])- qsym(y[0],y[1],2.0);
}

void runAPLP1F() {
	double xmin[3]= {2,2,2};
	double eM = getEdgeMax();
	double xmax[3]= {eM,eM,eM};
	Minimizer M(40,3,0,xmin,xmax);
	M.func = testAPLP1F;
	cout << "testAPLP1F:" << endl;
	M.coutReport();
	cout << "end-testAPLP1F" << endl << endl;
}

void testAPLP1G(int,int,double* y,double* ret,void*) {
	double twopi7 = 2.0*atan(1.0)*4.0/7.0;
	*ret = 2.0*(qsym(y[0],y[1],2)-qsym(2,y[1],2)) -L(y[0])
		- twopi7*(A(y[0]/2.0)-A(1.0));
}

void runAPLP1G() {
	double xmin[2]= {2,2};
	double eM = getEdgeMax();
	double xmax[2]= {eM,eM};
	Minimizer M(40,2,0,xmin,xmax);
	M.func = testAPLP1G;
	cout << "testAPLP1G:" << endl;
	M.coutReport();
	cout << "end-testAPLP1G" << endl << endl;
}

/***
  *  EXPERIMENTAL INEQUALITIES
  */
double deltaY(double y1,double y2,double y3,double y4,double y5,double y6) {
	return delta(y1*y1,y2*y2,y3*y3,y4*y4,y5*y5,y6*y6);
}

void constraintE(int,int which,double* y,double* ret,void*) {
	switch (which) {
	case 1: 
		*ret = y[4]-y[0]-y[2];
		break;
	case 2:
		*ret = y[3]-y[1]-y[2];
		break;
	case 3:
		*ret = y[3]-y[7]-y[8];
		break;
	case 4:
		*ret = y[3]-y[5]-y[4];
		break;
	case 5:
		// Should be 0.0, rather than 3.0, but problems without the eps.
	   *ret= -deltaY(y[0],y[1],y[2],y[3],y[4],y[5])+3.0;
	   break;
	case 6:
	   *ret= -deltaY(y[3],y[7],y[4],3.2,y[5],y[8]);
	   break;
	default:
		*ret=-deltaY(y[6],y[1],y[2],y[3],y[7],y[8]);
		break;
	/*
	case 8:
		*ret = -crossdiag(y) + 3.2;
		break;
	default:
		*ret = -deltaY(y[1],y[0],y[6],crossdiag(y),y[8],y[5]);
	*/
	}
}

double delta5(double x1,double x2,double x3,double x4,double x5,double x6) {
	return delta4(x2,x3,x1,x5,x6,x4);
}

double delta6(double x1,double x2,double x3,double x4,double x5,double x6) {
	return delta4(x3,x1,x2,x6,x4,x5);
}


double Sbar(double x1,double x2,double x3,double x4,double x5,double x6,
			double x7,double x8,double x9) {
	double deltaX = delta(x1,x2,x3,x4,x5,x6);
	double deltaY = delta(x7,x2,x3,x4,x8,x9);
	double de6 = delta6(x1,x2,x3,x4,x5,x6);
	double de5 = delta5(x1,x2,x3,x4,x5,x6);
	double de8 = delta5(x7,x2,x3,x4,x8,x9);
	double de9 = delta6(x7,x2,x3,x4,x8,x9);
	double u = uT(x2,x3,x4);
	double deriv = 
		 safesqrt(deltaY)*(B(x1)*sqrt(x1)*u - B(x2)*sqrt(x2)*de6 -B(x3)*sqrt(x3)*de5)
		+safesqrt(deltaX)*(B(x7)*sqrt(x7)*u - B(x2)*sqrt(x2)*de9 -B(x3)*sqrt(x3)*de8);
	return deriv;
}

double SSbar(double x1,double x2,double x3,double x4,double x5,double x6,
			 double x7,double x8,double x9) {
	double deltaX = delta(x1,x2,x3,x4,x5,x6);
	double deltaY = delta(x7,x2,x3,x4,x8,x9);
	double de6 = delta6(x1,x2,x3,x4,x5,x6);
	double de5 = delta5(x1,x2,x3,x4,x5,x6);
	double de8 = delta5(x7,x2,x3,x4,x8,x9);
	double de9 = delta6(x7,x2,x3,x4,x8,x9);
	double deltaX4 = delta4(x1,x2,x3,x4,x5,x6);
	double deltaY4 = delta4(x7,x2,x3,x4,x8,x9);
	double u = uT(x2,x3,x4);
	double u4 = 2.0*(-x4+x2+x3);
	double de64 = x1+x3-x5;
	double de54 = x1+x2-x6;
	double de84 = x7+x2-x9;
	double de94 = x7+x3-x8;
	double xi1 = B(x1)*sqrt(x1)*u - B(x2)*sqrt(x2)*de6-B(x3)*sqrt(x3)*de5;
	double xi2 = B(x7)*sqrt(x7)*u - B(x2)*sqrt(x2)*de9-B(x3)*sqrt(x3)*de8;
	double xi1Dx4= B(x1)*sqrt(x1)*u4 - B(x2)*sqrt(x2)*de64-B(x3)*sqrt(x3)*de54;
	double xi2Dx4 =B(x7)*sqrt(x7)*u4 - B(x2)*sqrt(x2)*de94-B(x3)*sqrt(x3)*de84;
	double deriv = u*safesqrt(deltaX)*(0.5*deltaY4*xi1 + deltaY*xi1Dx4)
				+  u*safesqrt(deltaY)*(0.5*deltaX4*xi2 + deltaX*xi2Dx4);
	return deriv;
}

double testE1param = 10.0;
void testE1(int,int,double* y,double* ret,void*) {
	double x1 = y[0]*y[0];
	double x2 = y[1]*y[1];
	double x3 = y[2]*y[2];
	double x4 = y[3]*y[3];
	double x5 = y[4]*y[4];
	double x6 = y[5]*y[5];
	double x7 = y[6]*y[6];
	double x8 = y[7]*y[7];
	double x9 = y[8]*y[8];
	*ret= testE1param*pow(Sbar(x1,x2,x3,x4,x5,x6,x7,x8,x9),2)
			-SSbar(x1,x2,x3,x4,x5,x6,x7,x8,x9);
}



void testCodeE1(double x1,double x2,double x3,double x4,
				double x5,double x6,double x7,double x8,double x9) {
	cout.precision(16);
	cout << "Sbar = " << Sbar(x1,x2,x3,x4,x5,x6,x7,x8,x9) << endl;
	cout << "SSbar = " << SSbar(x1,x2,x3,x4,x5,x6,x7,x8,x9) << endl;

}

void runE1() {
	double eM = getEdgeMax();
	double xmin[9]= {2,2,2,  sqrt(8.),sqrt(8.),2,  2,2,2};
	double xmax[9]= {eM,eM,eM,  3.201,2.0*eM,eM,  eM,2,2};
	Minimizer M(500,9,1,xmin,xmax);
	M.func = testE1;
	testE1param=10.0;
	M.cFunc = constraintDelta;
	cout << "testE1:" << endl;
	M.coutReport();
	cout << "end-testE1" << endl << endl;
}

void runE2() {
	double eM = getEdgeMax();
	double xmin[9]= {2,2,2,  3.2,1.44/*delta(top)>0*/,2,  2,2,2};
	double xmax[9]= {eM,eM,eM,  2.0*eM,2.0*eM,eM,  eM,eM,eM};
	Minimizer M(100,9,7/*constraints*/,xmin,xmax);
	M.func = testE1;
	testE1param=100.0;
	M.cFunc = constraintE;
	cout << "testE2:" << endl;
	M.coutReport();
	cout << "cross-diag " << crossdiag(M.x) << endl;
	double x1=M.x[0]*M.x[0];
	double x2=M.x[1]*M.x[1];
	double x3=M.x[2]*M.x[2];
	double x4=M.x[3]*M.x[3];
	double x5=M.x[4]*M.x[4];
	double x6=M.x[5]*M.x[5];
	double x7=M.x[6]*M.x[6];
	double x8=M.x[7]*M.x[7];
	double x9=M.x[8]*M.x[8];
	testCodeE1(x1,x2,x3,x4,x5,x6,x7,x8,x9);
	cout << "end-testE2" << endl << endl;
}



void runCH() {
	/*
	runCH25A1();
	runCH25A2();
	runCH25A3();
	runCH25A4();
	runCH26A();
	runCH26B();
	// chapter 3:
	runCH32A();
	runCH34C();
	runCH351();
	runCH35_234();
	runCH36A();
	testCodeCH36B();
	runCH36B();
	runCH36C();
	runCH376(); // FALSE!!
	runCH377();
	runCH37C();
	// chapter 4:
	runCH43B5();
	runCH43B6();
	runCH43C1();
	runCH43C2();
	runCH43C5();
	runCH47B();
	runCH47C();
	runCH48B();
	runCH48C();
	runCH48D();
	// chapter 6:
	runCH62E();
	runCH62F();
	runCH62G();
	runCH62H();
	// appendix (linear programming)
	runAPLP1C();
	runAPLP1D();
	runAPLP1F();
	runAPLP1G();
	// experimental:
	testCodeE1(4.1,4.2,4.3,4.4,4.5,4.6,4.7.4.8.4.9);
	runE1();
	*/
	//runE2();
	/*
	*/
	runXX();
}
