#include "Minimizer.h"
#include "kepfn.h"
#include <iomanip.h>

/** 
  * TESTS of multisimplex optimizations
  */


/**
  * sample problem....
  */
void testSAMPLE(int,int,double* y,double* ret,void*) {
	*ret = taut(y[0],y[1],y[2],y[3],y[4],y[5]) +
		   taut(y[6],y[1],y[2],y[3],y[7],y[8]) 
		   - (L(y[0])+L(y[1])+L(y[2])+L(y[6]))
			- 0.03464;
}

void constraintSAMPLE(int,int,double* y,double* ret,void*) {
	*ret = -crossdiag(y) + safesqrt(8.0);
}

void runSAMPLE() {
	double eM = getEdgeMax();
	double xmin[9]= {2,2,2,sqrt(8.0),2,2,  2,2,2};
	double xmax[9]= {eM,eM,eM,3.0,eM,eM,   2,2,2};
	Minimizer M(20,9,1,xmin,xmax);
	M.func = testSAMPLE;
	M.cFunc = constraintSAMPLE;
	cout << "testSAMPLE:" << endl;
	M.coutReport();
	cout << "crossdiag = " << crossdiag(M.x) << endl;
	cout << "end-testSAMPLE" << endl << endl;
}

/**
  * The configuration of 6 qrtets around two adjacent vertices (p,q)=(4,0).
  * 17 variables.
  */

double deltaL(double a, double b, double c) {
	return -L(a) - L(b) + 2.0*L(c);
}

double deltaNeg(double a, double b, double c) {
	return -L(a) - L(b);
}

double tat(int i1, int i2, int i3, int i4, int i5, int i6,double* y) {
	return taut(y[i1],y[i2],y[i3],y[i4],y[i5],y[i6]);
}

double di(int i1,int i2,int i3,int i4,int i5,int i6,double* y) {
	return dih(y[i1],y[i2],y[i3],y[i4],y[i5],y[i6]);
}

void test6qrtet(int,int,double* y,double* ret,void*) {
	*ret = +tat(0,1,2,13,16,6,y)
			+tat(1,3,2,12,13,7,y)
			+tat(2,4,3,11,12,14,y)
			+tat(3,4,5,10,8,11,y)
			+tat(5,0,4,15,10,9,y)
			+tat(0,2,4,14,15,16,y)
	// perimeter qr triangles
	+deltaL(y[0],y[1],y[6])/2.0
		+deltaL(y[1],y[3],y[7])/2.0
		+deltaL(y[3],y[5],y[8])/2.0
		+deltaL(y[5],y[0],y[9])/2.0
	// internal qr triangles
	+deltaL(y[1],y[2],y[13])
	+deltaL(y[3],y[2],y[12])
	+deltaL(y[4],y[2],y[14])
	+deltaL(y[0],y[2],y[16])
	+deltaL(y[3],y[4],y[11])
	+deltaL(y[5],y[4],y[10])
	+deltaL(y[0],y[4],y[15]);
}

void constraint6qrtet(int,int which,double* y,double* ret,void*) {
	static const double pi = 4.0*atan(1.0);
	*ret = 0.0;
	if (which==1) {
	*ret= -di(2,1,0,6,16,13,y)-di(2,1,3,7,12,13,y)
			-di(2,3,4,11,14,12,y)-di(2,4,0,15,16,14,y)+2.0*pi;
	}
	else {
	*ret = -di(4,2,3,12,11,14,y)-di(4,3,5,8,10,11,y)
		-di(4,5,0,9,15,10,y)-di(4,2,0,16,15,14,y)+2.0*pi;
	}
}

void run6qrtet() {
	double eM = getEdgeMax();
	double xmin[17];
	double xmax[17];
	for (int i=0;i<17;i++) { xmin[i]=2.0; xmax[i]=eM; }
	Minimizer M(20,17,2,xmin,xmax);
	M.func = test6qrtet;
	M.cFunc = constraint6qrtet;
	cout << "test6qrtet:" << endl;
	M.coutReport();
	cout << "end-test6qrtet" << endl << endl;
}

/**
  * The configuration of a pentagon and triangle merging into a quad
  */
void testPentTri(int,int,double* y,double* ret,void*) {
	*ret = +tat(2,1,0,5,4,3,y)+tat(2,1,7,6,8,3,y)
		+tat(2,7,9,12,10,8,y)+tat(2,9,0,11,4,10,y)
		-2.0*L(y[2])-L(y[9])-L(y[7])-2.0*L(y[1])-2.0*L(y[0]);
}

void constraintPentTri(int,int which,double* y,double* ret,void*) {
	static const double pi = 4.0*atan(1.0);
	*ret= -di(2,1,0,5,4,3,y)-di(2,1,7,6,8,3,y)
		-di(2,7,9,12,10,8,y)-di(2,9,0,11,4,10,y)+2.0*pi;
}

void runPentTri() {
	double eM = getEdgeMax();
	double xmin[13];
	double xmax[13];
	for (int i=0;i<13;i++) { xmin[i]=2.0; xmax[i]=eM; }
	xmin[8]=xmin[10]=eM;
	xmax[8]=xmax[10]=3.2; // could test farther out, but this is only a test.
	Minimizer M(20,13,1,xmin,xmax);
	M.func = testPentTri;
	M.cFunc = constraintPentTri;
	cout << "testPentTri:" << endl;
	M.coutReport();
	cout << "end-testPentTri" << endl << endl;
}

/**
  * The configuration of a hexagon folded into a quad.
  */

void testHexInQuad(int,int,double* y,double* ret,void*) {
	*ret = +tat(2,1,0,5,4,3,y)+tat(2,1,7,6,8,3,y)
		+tat(2,7,9,12,10,8,y)+tat(2,9,0,11,4,10,y)
		-2.0*L(y[2])-L(y[9])-L(y[7])-L(y[1])-2.0*L(y[0]);
}

void constraintHexInQuad(int,int which,double* y,double* ret,void*) {
	static const double pi = 4.0*atan(1.0);
	*ret= -di(2,1,0,5,4,3,y)-di(2,1,7,6,8,3,y)
		-di(2,7,9,12,10,8,y)-di(2,9,0,11,4,10,y)+2.0*pi;
}

void runHexInQuad() {
	double eM = getEdgeMax();
	double xmin[13];
	double xmax[13];
	for (int i=0;i<13;i++) { xmin[i]=2.0; xmax[i]=eM; }
	xmin[8]=xmin[10]=xmin[3]=eM;
	xmax[8]=xmax[10]=xmax[3]=3.2; // could test farther out, but this is only a test.
	Minimizer M(20,13,1,xmin,xmax);
	M.func = testHexInQuad;
	M.cFunc = constraintHexInQuad;
	cout << "testHexInQuad:" << endl;
	M.coutReport();
	cout << "end-testHexInQuad" << endl << endl;
}

/**
  * The configuration of two quads combining to a quad (2 shared edges).
  */

void test2Quad(int,int,double* y,double* ret,void*) {
	*ret = +tat(2,1,0,5,4,3,y)+tat(2,1,7,6,8,3,y)
		+tat(2,7,9,12,10,8,y)+tat(2,9,0,11,4,10,y)
		-L(y[2])-L(y[9])-2.0*L(y[7])-L(y[1])-2.0*L(y[0]);
}

void constraint2Quad(int,int which,double* y,double* ret,void*) {
	static const double pi = 4.0*atan(1.0);
	*ret= -di(2,1,0,5,4,3,y)-di(2,1,7,6,8,3,y)
		-di(2,7,9,12,10,8,y)-di(2,9,0,11,4,10,y)+2.0*pi;
}

void run2Quad() {
	double eM = getEdgeMax();
	double xmin[13];
	double xmax[13];
	for (int i=0;i<13;i++) { xmin[i]=2.0; xmax[i]=eM; }
	xmin[10]=xmin[3]=eM;
	xmax[10]=xmax[3]=3.2; // could test farther out, but this is only a test.
	Minimizer M(20,13,1,xmin,xmax);
	M.func = test2Quad;
	M.cFunc = constraint2Quad;
	cout << "test2Quad:" << endl;
	M.coutReport();
	cout << "end-test2Quad" << endl << endl;
}

/**
  * test of tau(p,q) for crowded p,q with 4 guys around a vertex *
  * c1,...,c4 are boolean, +1 means it is a quad. 0 means it is a triangle.
  */
int c2,c4,c6,c8;
double tatb(int c,int i1,int i2,int i3,int i4,int i5,int i6,double* y) {
	return (c>0 ? tat(i1,i2,i3,i4,i5,i6,y) : 0.0);
}

double del(int c,int i1,int i2,int i3,double*y) {
	return (c>0? deltaNeg(y[i1],y[i2],y[i3]) : deltaL(y[i1],y[i2],y[i3]));
}

void test4pq(int,int,double* y,double* ret,void*) {
	double nquad = (double) (c2+c4+c6+c8);
	double ntri = 4.0-nquad;
	double excessT = 0.00051; // bound on VolAn(2^6)-Volt(2^6).
	*ret = tat(0,1,7,21,24,9,y)+tatb(c8,8,1,7,21,22,23,y)
			+tat(0,1,3,10,13,9,y)+tatb(c2,2,1,3,10,12,11,y)
			+tat(0,3,5,14,17,13,y)+tatb(c4,4,3,5,14,16,15,y)
			+tat(0,5,7,18,24,17,y)+tatb(c6,6,5,7,18,20,19,y)
			+deltaL(y[0],y[1],y[9])+deltaL(y[0],y[3],y[13])
			+deltaL(y[0],y[5],y[17])+deltaL(y[0],y[7],y[24])
			-(c2>0?L(y[2]):0.0)-(c4>0?L(y[4]):0.0)
			-(c6>0?L(y[6]):0.0)-(c8>0?L(y[8]):0.0)
			+(del(c2,1,3,10,y)+del(c4,3,5,14,y)+del(c6,5,7,18,y)+del(c8,7,1,21,y))/2.0
			-2.0*nquad*M(getEdgeMax())
			-excessT*ntri;
}

void constraint4pq(int,int,double* y,double* ret,void*) {
	double pi = 4.0*atan(1.0);
	*ret = -2.0*pi + di(0,1,7,21,24,9,y)+di(0,1,3,10,13,9,y)
			+di(0,3,5,14,17,13,y)+di(0,5,7,18,24,17,y);
}

void run4pq() {
	double eM = getEdgeMax();
	double s8 = sqrt(8.0);
	double xmin[25];
	double xmax[25];

	double tau[5][5];
	{ for (int i=0;i<5;i++) for (int j=0;j<5;j++) { tau[i][j]= 1.0; } }
	for (c2=0;c2<=1;c2++)
	for (c4=0;c4<=1;c4++)
	for (c6=0;c6<=1;c6++)
	for (c8=0;c8<=1;c8++)
	{
	/*init*/ { for (int i=0;i<25;i++) { xmin[i]=2.0; xmax[i]=eM; }}
		if (c2>0) { xmin[10]=eM; xmax[10]=s8; }
		if (c4>0) { xmin[14]=eM; xmax[14]=s8; }
		if (c6>0) { xmin[18]=eM; xmax[18]=s8; }
		if (c8>0) { xmin[21]=eM; xmax[21]=s8; }
	int nquad = c2+c4+c6+c8;
	int ntri  = 4-nquad;
	Minimizer M(20,25,1,xmin,xmax);
	M.func = test4pq;
	M.cFunc = constraint4pq;
	cout << "test4pq:" << endl;
	cout << ntri << " && " << nquad << endl;
	M.coutReport();
	double value;
	M.func(0,0,M.x,&value,0);
	if (value < tau[ntri][nquad]) 
		tau[ntri][nquad]=value;
	}
	for (int i=0;i<5;i++) for (int j=0;j<5;j++) 
		if (tau[i][j]<1.0)
			cout << "tau("<<i<<","<<j<<")= "<< tau[i][j] << endl;
	cout << "end-test4pq" << endl << endl;
}

/**
  * test of tau(p,q) for crowded p,q with 5 guys around a vertex *
  * c1,...,c4 are boolean, +1 means it is a quad. 0 means it is a triangle.
  */
int /* c2,c4,c6,c8, */ c10, c12;

void test5pq(int,int,double* y,double* ret,void*) {
	double nquad = (double) (c2+c4+c6+c8+c10);
	double ntri = 5.0-nquad;
	double excessT = 0.00051; // bound on VolAn(2^6)-Volt(2^6).
	*ret = tat(0,1,3,16,12,11,y)+tatb(c2,2,1,3,16,23,22,y)
			+tat(0,3,5,17,13,12,y)+tatb(c4,4,3,5,17,25,24,y)
			+tat(0,5,7,18,14,13,y)+tatb(c6,6,5,7,18,27,26,y)
			+tat(0,7,9,19,15,14,y)+tatb(c8,8,7,9,19,29,28,y)
			+tat(0,9,1,20,11,15,y)+tatb(c10,10,1,9,20,30,21,y)
			-(c2>0?L(y[2]):0.0)-(c4>0?L(y[4]):0.0)-(c6>0?L(y[6]):0.0)
			-(c8>0?L(y[8]):0.0)-(c10>0?L(y[10]):0.0)
			+deltaL(y[0],y[1],y[11])+deltaL(y[0],y[3],y[12])
			+deltaL(y[0],y[5],y[13])+deltaL(y[0],y[7],y[14])
			+deltaL(y[0],y[9],y[15])
			+(del(c2,1,3,16,y)+del(c4,3,5,17,y)+del(c6,5,7,18,y)+
			  del(c8,7,9,19,y)+del(c10,9,1,20,y))/2.0
			-2.0*nquad*M(getEdgeMax())
			-excessT*ntri;
}

void constraint5pq(int,int which,double* y,double* ret,void*) {
	double pi = 4.0*atan(1.0);
	double angle = -2.0*pi + 
		di(0,1,3,16,12,11,y)
			+di(0,3,5,17,13,12,y)
			+di(0,5,7,18,14,13,y)
			+di(0,7,9,19,15,14,y)
			+di(0,9,1,20,11,15,y);
	*ret = angle;
		;
}

void run5pq() {
	double eM = getEdgeMax();
	double s8 = sqrt(8.0);
	double xmin[31];
	double xmax[31];
	double* y;
	int Nvar = 31;

	double tau[7][7];
	{ for (int i=0;i<7;i++) for (int j=0;j<7;j++) { tau[i][j]= 10.0; } }
	for (c2=0;c2<=1;c2++)
	for (c4=0;c4<=1;c4++)
	for (c6=0;c6<=1;c6++)
	for (c8=0;c8<=1;c8++)
	for (c10=0;c10<=1;c10++)
	{
	c12=0;
	/*init*/ { for (int i=0;i<Nvar;i++) { xmin[i]=2.0; xmax[i]=eM; }}
		if (c2>0) { xmin[16]=eM; xmax[16]=s8; }
		if (c4>0) { xmin[17]=eM; xmax[17]=s8; }
		if (c6>0) { xmin[18]=eM; xmax[18]=s8; }
		if (c8>0) { xmin[19]=eM; xmax[19]=s8; }
		if (c10>0) { xmin[20]=eM; xmax[20]=s8; }
	int nquad = c2+c4+c6+c8+c10+c12;
	int ntri  = 5-nquad;
	Minimizer M(50,Nvar,1,xmin,xmax);
	M.func = test5pq;
	M.cFunc = constraint5pq;
	cout << "test5pq:" << endl;
	cout << ntri << " && " << nquad << endl;
	M.coutReport();
	double value;
	M.func(0,0,M.x,&value,0);
	if (value < tau[ntri][nquad]) 
		tau[ntri][nquad]=value;
	y = M.x;
	}
	for (int i=0;i<7;i++) for (int j=0;j<7;j++) 
		if (tau[i][j]<1.0)
			cout << "tau("<<i<<","<<j<<")= "<< tau[i][j] << endl;
	cout << "end-test5pq" << endl << endl;
}

/**
  * test of tau(p,q) for crowded p,q with 6 guys around a vertex *
  * c1,...,c4 are boolean, +1 means it is a quad. 0 means it is a triangle.
  */

void test6pq(int,int,double* y,double* ret,void*) {
	double nquad = (double) (c2+c4+c6+c8);
	double ntri = 6.0-nquad;
	double excessT = 0.00051; // bound on VolAn(2^6)-Volt(2^6).
	*ret = tat(0,1,3,19,14,13,y)+tatb(c2,2,1,3,19,26,25,y)
			+tat(0,3,5,20,15,14,y)+tatb(c4,4,3,5,20,28,27,y)
			+tat(0,5,7,21,16,15,y)+tatb(c6,6,5,7,21,30,29,y)
			+tat(0,7,9,22,17,16,y)+tatb(c8,8,7,9,22,32,31,y)
			+tat(0,9,11,23,18,17,y)+tatb(c10,10,9,11,23,34,33,y)
			+tat(0,11,1,24,13,18,y)+tatb(c12,12,11,1,24,36,35,y)
			-(c2>0?L(y[2]):0.0)-(c4>0?L(y[4]):0.0)-(c6>0?L(y[6]):0.0)
			-(c8>0?L(y[8]):0.0)-(c10>0?L(y[10]):0.0)
			-(c12>0?L(y[12]):0.0)
			+deltaL(y[0],y[1],y[13])+deltaL(y[0],y[3],y[14])
			+deltaL(y[0],y[5],y[15])+deltaL(y[0],y[7],y[16])
			+deltaL(y[0],y[9],y[17])+deltaL(y[0],y[11],y[18])
			+(del(c2,1,3,19,y)+del(c4,3,5,20,y)+del(c6,5,7,21,y)+
			  del(c8,7,9,22,y)+del(c10,9,11,23,y)+del(c12,11,1,24,y))/2.0
			-2.0*nquad*M(getEdgeMax())
			-excessT*ntri;
}

void constraint6pq(int,int,double* y,double* ret,void*) {
	double pi = 4.0*atan(1.0);
	*ret = -2.0*pi + 
		di(0,1,3,19,14,13,y)
			+di(0,3,5,20,15,14,y)
			+di(0,5,7,21,16,15,y)
			+di(0,7,9,22,17,16,y)
			+di(0,9,11,23,18,17,y)
			+di(0,11,1,24,13,18,y)
		;
}

void run6pq() {
	double eM = getEdgeMax();
	double s8 = sqrt(8.0);
	double xmin[37];
	double xmax[37];
	int Nvar = 37;

	double tau[7][7];
	{ for (int i=0;i<7;i++) for (int j=0;j<7;j++) { tau[i][j]= 10.0; } }
	for (c2=0;c2<=1;c2++)
	for (c4=0;c4<=1;c4++)
	for (c6=0;c6<=1;c6++)
	for (c8=0;c8<=1;c8++)
	for (c10=0;c10<=1;c10++)
	for (c12=0;c12<=1;c12++)
	{
	/*init*/ { for (int i=0;i<Nvar;i++) { xmin[i]=2.0; xmax[i]=eM; }}
		if (c2>0) { xmin[19]=eM; xmax[19]=s8; }
		if (c4>0) { xmin[20]=eM; xmax[20]=s8; }
		if (c6>0) { xmin[21]=eM; xmax[21]=s8; }
		if (c8>0) { xmin[22]=eM; xmax[22]=s8; }
		if (c10>0) { xmin[23]=eM; xmax[23]=s8; }
		if (c12>0) { xmin[24]=eM; xmax[24]=s8; }
	int nquad = c2+c4+c6+c8+c10+c12;
	int ntri  = 6-nquad;
	Minimizer M(50,Nvar,1,xmin,xmax);
	M.func = test6pq;
	M.cFunc = constraint6pq;
	cout << "test6pq:" << endl;
	if (ntri<=1) continue;
	cout << ntri << " && " << nquad << endl;
	M.coutReport();
	double value;
	M.func(0,0,M.x,&value,0);
	if (value < tau[ntri][nquad]) 
		tau[ntri][nquad]=value;
	}
	for (int i=0;i<7;i++) for (int j=0;j<7;j++) 
		if (tau[i][j]<10.0)
			cout << "tau("<<i<<","<<j<<")= "<< tau[i][j] << endl;
	cout << "end-test6pq" << endl << endl;
}

/**
  * test of tau(p,q) for crowded p,q with 7 guys around a vertex *
  * c1,...,c4 are boolean, +1 means it is a quad. 0 means it is a triangle.
  */

void test7pq(int,int,double* y,double* ret,void*) {
	double nquad = (double) c2;
	double ntri = 7.0-nquad;
	double excessT = 0.00051; // bound on VolAn(2^6)-Volt(2^6).
	*ret = tat(0,1,3,16,10,9,y)+tatb(c2,2,1,3,16,24,23,y)
			+tat(0,3,4,17,11,10,y)
			+tat(0,4,5,18,12,11,y)
			+tat(0,5,6,19,13,12,y)
			+tat(0,6,7,20,14,13,y)
			+tat(0,7,8,21,15,14,y)
			+tat(0,8,1,22,9,15,y)
			-(c2>0?L(y[2]):0.0)
			+deltaL(y[0],y[1],y[9])+deltaL(y[0],y[3],y[10])
			+deltaL(y[0],y[4],y[11])+deltaL(y[0],y[5],y[12])
			+deltaL(y[0],y[6],y[13])+deltaL(y[0],y[7],y[14])
			+deltaL(y[0],y[8],y[15])
			+(del(c2,1,3,16,y))/2.0
			-2.0*nquad*M(getEdgeMax())
			-excessT*ntri;
}

void constraint7pq(int,int,double* y,double* ret,void*) {
	double pi = 4.0*atan(1.0);
	*ret = -2.0*pi + 
		di(0,1,3,16,10,9,y)
			+di(0,3,4,17,11,10,y)
			+di(0,4,5,18,12,11,y)
			+di(0,5,6,19,13,12,y)
			+di(0,6,7,20,14,13,y)
			+di(0,7,8,21,15,14,y)
			+di(0,8,1,22,9,15,y);
		
}

void run7pq() {
	double eM = getEdgeMax();
	double s8 = sqrt(8.0);
	double xmin[25];
	double xmax[25];
	int Nvar = 25;

	double tau[7][7];
	{ for (int i=0;i<7;i++) for (int j=0;j<7;j++) { tau[i][j]= 100.0; } }
	for (c2=0;c2<=0;c2++)
	{
	/*init*/ { for (int i=0;i<Nvar;i++) { xmin[i]=2.0; xmax[i]=eM; }}
		if (c2>0) { xmin[16]=eM; xmax[16]=s8; }
	int nquad = c2;
	int ntri  = 7-nquad;
	Minimizer M(50,Nvar,1,xmin,xmax);
	M.func = test7pq;
	M.cFunc = constraint7pq;
	cout << "test7pq:" << endl;
	cout << ntri << " && " << nquad << endl;
	M.coutReport();
	double value;
	M.func(0,0,M.x,&value,0);
	if (value < tau[ntri][nquad]) 
		tau[ntri][nquad]=value;
	}
	for (int i=0;i<7;i++) for (int j=0;j<7;j++) 
		if (tau[i][j]<100.0)
			cout << "tau("<<i<<","<<j<<")= "<< tau[i][j] << endl;
	cout << "end-test7pq" << endl << endl;
}

void runCH() {
	/*
	runSAMPLE(); // sample problem.
	run6qrtet();
	runPentTri();
	runHexInQuad();
	run2Quad();
	run4pq();
	run5pq();
	run6pq();
	run7pq();
	*/
	run5pq();

}
