#include "kepfn.h"

double getEdgeMax() {
	//static const double edgeMax =2.558913201377738; 
	static const double edgeMax = 2.5589;
	return edgeMax;
}

double getTrunc() {
	static const double trunc =getEdgeMax()/2.0;
	return trunc;
}

double getDtet() {
	static const double dtet = sqrt(8.0)*atan(sqrt(2.0)/5.0);
	return dtet;
}

double L(double c) {
	return 0.035*(c-2.0) - 0.06*(c-2.0)*(c-getEdgeMax());
}

double M(double y) {
	static const double m = 0.006;
	return m*(sqrt(8.0)-y);
}

double safesqrt(double t)
	{
	if (t<0.0) return 0.0; else return sqrt(t);
	}

double delta(double x1,double x2,double x3,double x4,double x5,double x6)
        {
        return (-x1+x2+x3-x4+x5+x6)*(x1*x4) +
                (x1-x2+x3+x4-x5+x6)*(x2*x5)+
                (x1+x2-x3+x4+x5-x6)*(x3*x6) -
                (x2*x3*x4 +x1*x3*x5+x1*x2*x6+x4*x5*x6);
        }

double dihedralX(double x1, double x2, double x3, double x4, double x5,
	double x6)  {
    double d4 = -x1*x1 + x1*x2 + x1*x3 - x2*x3 - 2.0*x1*x4 + 
	    x1*x5 + x2*x5 + x1*x6 + x3*x6 - x5*x6; 
    return atan(1.0)*2.0 + 
	    atan(-(d4/sqrt(4.0*x1*delta(x1, x2, x3, x4, x5, x6))))  ;   
}

double dih(double y1,double y2,double y3,double y4,double y5,
        double y6) {
        return dihedralX(y1*y1,y2*y2,y3*y3,y4*y4,y5*y5,y6*y6);
}

double solid(double y1,double y2,double y3,double y4,double y5,
	double y6) {
	return 
	dih(y1,y2,y3,y4,y5,y6)+dih(y2,y3,y1,y5,y6,y4) +
	dih(y3,y1,y2,y6,y4,y5)- atan(1.0)*4.0;   
}

double U(double a,double b,double c) {
    return -a*a-b*b-c*c+2.0*(a*b+b*c+c*a);
}

double uT(double x1,double x2,double x3) {
	return U(x1,x2,x3);
}

double eta2X(double x1,double x2,double x3) {
        return x1*x2*x3/U(x1,x2,x3);
        }

double eta(double y1,double y2,double y3) {
        return safesqrt(eta2X(y1*y1,y2*y2,y3*y3));
        }
double chiX(   double x1,double x2,double x3,
                double x4,double x5,double x6) {
        return x3*x6*(x4+x5-x6) + x2*x5*(x4-x5+x6)+x1*x4*(-x4+x5+x6)
               -2.0*x4*x5*x6;
        }

double radX(double x1,double x2,double x3,double x4,
                double x5,double x6) {
        double c = chiX(x1,x2,x3,x4,x5,x6);
        return safesqrt(eta2X(x4,x5,x6)+
                        c*c/(4.0*U(x4,x5,x6)*delta(x1,x2,x3,x4,x5,x6)));
        }

double rad(double y1,double y2,double y3,double y4,double y5,double y6) {
	return radX(y1*y1,y2*y2,y3*y3,y4*y4,y5*y5,y6*y6);
	}

double quoin(double a,double b,double c) {
	if ((a>b)||(b>c)) return 0;
	double u = sqrt((c*c - b*b)/(b*b - a*a));
	return 
	-(a*a + a*c - 2*c*c)*(c - a)*atan(u)/6.0 +
	(a*(b*b - a*a)*u)/6.0 - (2.0/3.0)*c*c*c*atan((u*(b - a))/(b + c));
}

double qy(double y1, double y2, double y3) {
	 return quoin(y1/2.0, eta(y1,y2,y3), getTrunc());  
}

double qy(double y1, double y2, double y3, double trunc) {
	 // added to stabilize quoin calculation near a==b
	 if (y1/2.0 >= eta(y1,y2,y3) - 1.0e-6) return 0.0;
	 if (eta(y1,y2,y3) > trunc - 1.0e-6) return 0.0;
	 return quoin(y1/2.0, eta(y1,y2,y3), trunc);  
}

double arc(double y1,double y2,double y3) {
	return acos( (y1*y1+y2*y2-y3*y3)/(2.0*y1*y2) );
}

double beta(double psi,double theta) {
	double cpsi = cos(psi);
	double ctheta = cos(theta);
	double rhs = (pow(cpsi,2)-pow(ctheta,2))/(1.0-pow(ctheta,2));
	return acos(safesqrt(rhs));
}

double phi(double h, double t) {
	return  h*t*(h+t)/6.0;
}

double phi0() {
	return phi(getTrunc(),getTrunc());
}

double A(double h, double t) {
	return (1.0-h/t)*(phi(t,t)-phi(h,t));         
}

double A(double h) {
	return A(h, getTrunc());
}

double volt(double y1,double y2,double y3,double y4,double y5,double y6) {
  return  phi0()*solid(y1, y2, y3, y4, y5, y6) -
    A(y1/2.0)*dih(y1, y2, y3, y4, y5, y6) -
    A(y2/2.0)*dih(y2, y1, y3, y5, y4, y6) -
    A(y3/2.0)*dih(y3, y1, y2, y6, y4, y5) +
    qy(y1, y2, y6) + qy(y2, y1, y6) +
    qy(y3, y2, y4) + qy(y2, y3, y4) +
    qy(y1, y3, y5) + qy(y3, y1, y5);
}

double volq(double y1,double y2,double y3,double y4,double y5,double y6,double trunc) {
  return  phi(trunc,trunc)*solid(y1, y2, y3, y4, y5, y6) -
    A(y1/2.0,trunc)*dih(y1, y2, y3, y4, y5, y6) -
    A(y2/2.0,trunc)*dih(y2, y1, y3, y5, y4, y6) -
    A(y3/2.0,trunc)*dih(y3, y1, y2, y6, y4, y5) +
    qy(y1, y2, y6, trunc) + qy(y2, y1, y6, trunc) +
    qy(y3, y2, y4, trunc) + qy(y2, y3, y4, trunc) +
    qy(y1, y3, y5, trunc) + qy(y3, y1, y5, trunc);
}


double taut(double y1,double y2,double y3,double y4,double y5,double y6) {
	return volt(y1,y2,y3,y4,y5,y6) - solid(y1,y2,y3,y4,y5,y6)/(3.0*getDtet());
}

double volR(double a,double b,double c) {
	if (a>b || b>c) return 0.0;
	return sqrt(a*a*(b*b-a*a)*(c*c-b*b))/6.0;
}

double volan(double y1,double y2,double y3,double y4,double y5,double y6) {
	double r = rad(y1,y2,y3,y4,y5,y6);
	return 
	volR(y1/2.0, eta(y1, y2, y6), r) + volR(y2/2.0, eta(y1, y2, y6), r) +
	volR(y2/2.0, eta(y2, y3, y4), r) + volR(y3/2.0, eta(y2, y3, y4), r) +
	volR(y3/2.0, eta(y3, y1, y5), r) + volR(y1/2.0, eta(y3, y1, y5), r) ;
}

double crossdiag(double y[9])
	{
	double    x3=y[3]*y[3], 
		x4=y[4]*y[4], x5=y[5]*y[5], 
		x7=y[7]*y[7], x8=y[8]*y[8];
	double dd = dih(y[3],y[1],y[5],y[0],y[4],y[2])+
			dih(y[3],y[1],y[8],y[6],y[7],y[2]);
	static double pi = 3.14159265358979323;
	double c0;
	double c1 = -x3*x3-x5*x8-x4*x7+x3*(x5+x8+x4+x7)+x5*x7+x4*x8;
	if (dd>pi)
		{
		c0 = cos(2.0*pi-dd)*sqrt(U(x3,x5,x4)*(U(x3,x8,x7)));
		return sqrt((c0-c1)/(-2.0*x3));
		}
	else
		{
		c0 = cos(dd)*sqrt(U(x3,x5,x4)*(U(x3,x8,x7)));
		return  sqrt((c0-c1)/(-2.0*x3));
		}
	}