#include "error.h"
#include <float.h>
#include <iomanip.h>
#include "interval.h"
#include "lineInterval.h"
#include "secondDerive.h"
#include "taylorInterval.h"
#include "recurse.h"

void selfTest()
	{
	interMath::selfTest();
	linearization::selfTest();
	secondDerive::selfTest();
	taylorFunction::selfTest();
	}

void miniTest() {
	cout.precision(30);
	interval A("2.1");
	interval B("2.3");
	interval C = A*B;
	cout << C << endl;
	cout << interMath::atan(C) << endl;
	double x = 1.1;
	double y = 1.1;
	cout << "none " << x*y << endl;

	unsigned t;
	t = _control87(RC_DOWN,MCW_RC);
	cout << t << " " << x*y << endl;
	t = _control87(RC_UP,MCW_RC);
	cout << t << " " << x*y << endl;
	t = _control87(0x0800,MCW_RC);
	cout << t << " " <<  x*y << endl;
	x = 0.9;
	y = 0.9;
	t = _control87(0x0000,MCW_RC);
	cout.precision(16);
	cout << t << " " << x*y << endl;
	t = _control87(0x0400,MCW_RC);
	cout << t << " " << x*y << endl;
	t = _control87(0x0800,MCW_RC);
	cout << t << " " <<  x*y << endl;
	ROUND_UP;
	cout << x*y << endl;
	ROUND_DOWN;
	cout << x*y << endl;
}

//
// prove that dih > 1.153093 of a flat quarter.
int testRun()
	{
	interval tx[6]={"4","4","4","6.3001","4","4"};
	interval tz[6]={"6.3001","6.3001","6.3001","6.3001","6.3001","6.3001"};
	domain x = domain::lowerD(tx);
	domain z = domain::upperD(tz);
	taylorFunction F = taylorSimplex::dih*"-1"+taylorSimplex::unit*"1.153093";
	F.setReducibleState(1);
	return prove::generic (x,z,F);
	}


// prove that coeff*dih < a + b1 y1 + b2 y2 + ... b6 y6
// When yi varies from ymi to yMi.
// The values are the exact double values of the inequalities requested.
int proveDih(double coeff, double a, 
			double b1, double b2, double b3, double b4, double b5, double b6,
			double ym1, double ym2, double ym3, double ym4, double ym5, double ym6,
			double yM1, double yM2, double yM3, double yM4, double yM5, double yM6) {
	 if (!INTERVAL_ON) 
		 error::message("interval arithmetic turned off");
	interval C(coeff,coeff);
	interval A(a,a);
	interval B1(b1,b1), B2(b2,b2), B3(b3,b3), B4(b4,b4), B5(b5,b5), B6(b6,b6);
	interval Ym1(ym1,ym1), Ym2(ym2,ym2), Ym3(ym3,ym3), Ym4(ym4,ym4), Ym5(ym5,ym5), Ym6(ym6,ym6);
	interval YM1(yM1,yM1), YM2(yM2,yM2), YM3(yM3,yM3), YM4(yM4,yM4), YM5(yM5,yM5), YM6(yM6,yM6);
	interval tx[6]={Ym1*Ym1,Ym2*Ym2,Ym3*Ym3,Ym4*Ym4,Ym5*Ym5,Ym6*Ym6};
	interval tz[6]={YM1*YM1,YM2*YM2,YM3*YM3,YM4*YM4,YM5*YM5,YM6*YM6};
	domain x = domain::lowerD(tx);
	domain z = domain::lowerD(tz);
	taylorFunction F = taylorSimplex::dih*"-1"*C
		+taylorSimplex::unit*A
		+taylorSimplex::y1*B1
		+taylorSimplex::y2*B2
		+taylorSimplex::y3*B3
		+taylorSimplex::y4*B4
		+taylorSimplex::y5*B5
		+taylorSimplex::y6*B6;
	F.setReducibleState(0);
	if (!prove::generic (x,z,F)) return 0;
	//if (error::getCount()>0) return 0; // BUG add this in release version.
	return 1;
}



int main(int argc, char* argv[])
	{
	//miniTest();
	selfTest();
	cout.precision(20);
	if (testRun()) 
		cout << "YES!\n"; else cout << "NO!" ;
	cout << flush;
	if (proveDih(1,1.153,0,0,0,0,0,0,2,2,2,2.51,2,2,2.51,2.51,2.51,2.51,2.51,2.51))
		cout << "YES2\n" ; else cout << "NO2";
	cout << flush;
	if (proveDih(-1,-1.89,0,0,0,0,0,0,2,2,2,2,2,2,2.51,2.51,2.51,2.51,2.51,2.51))
		cout << "YES3\n"; else cout << "NO3";
	cout << "\nhello" << endl;
	return 0;
	}
